@@ -1,6 +1,6 @@
 /* mg-entry-combo.c
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2004 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,6 +20,7 @@
 
 #include "mg-entry-combo.h"
 #include <libmergeant/mg-server.h>
+#include <libmergeant/mg-server-data-type.h>
 #include <libmergeant/mg-data-handler.h>
 #include <libmergeant/mg-query.h>
 #include <libmergeant/mg-entity.h>
@@ -32,6 +33,7 @@
 #include <libmergeant/utility.h>
 
 #define USE_OLD_GTK_COMBO_WIDGET
+#undef USE_OLD_GTK_COMBO_WIDGET
 
 static void mg_entry_combo_class_init (MgEntryComboClass *class);
 static void mg_entry_combo_init (MgEntryCombo *wid);
@@ -49,7 +51,11 @@ static void mg_entry_combo_get_property (GObject              *object,
 static void          display_combo_model       (MgEntryCombo *combo);
 static void          choose_auto_default_value (MgEntryCombo *combo);
 #ifdef USE_OLD_GTK_COMBO_WIDGET
-static void          set_selection_combo_model (MgEntryCombo *combo, gint pos);
+static void          old_set_selection_combo_model (MgEntryCombo *combo, gint pos);
+static void          old_combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo);
+#else
+static void          new_set_selection_combo_model (MgEntryCombo *combo, gint pos);
+static void          new_combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo);
 #endif
 
 static void          mg_entry_combo_emit_signal (MgEntryCombo *combo);
@@ -166,18 +172,15 @@ mg_entry_combo_class_init (MgEntryComboClass *class)
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 }
 
-#ifdef USE_OLD_GTK_COMBO_WIDGET
-static void combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo);
-#endif
-
 static void
 real_combo_block_signals (MgEntryCombo *wid)
 {
 #ifdef USE_OLD_GTK_COMBO_WIDGET
 	g_signal_handlers_block_by_func (G_OBJECT (GTK_COMBO (wid->priv->combo_entry)->entry),
-					 G_CALLBACK (combo_contents_changed_cb), wid);
+					 G_CALLBACK (old_combo_contents_changed_cb), wid);
 #else
-	TO_IMPLEMENT;
+	g_signal_handlers_block_by_func (G_OBJECT (wid->priv->combo_entry),
+					 G_CALLBACK (new_combo_contents_changed_cb), wid);
 #endif
 }
 
@@ -186,9 +189,10 @@ real_combo_unblock_signals (MgEntryCombo *wid)
 {
 #ifdef USE_OLD_GTK_COMBO_WIDGET
 	g_signal_handlers_unblock_by_func (G_OBJECT (GTK_COMBO (wid->priv->combo_entry)->entry),
-					   G_CALLBACK (combo_contents_changed_cb), wid);
+					   G_CALLBACK (old_combo_contents_changed_cb), wid);
 #else
-	TO_IMPLEMENT;
+	g_signal_handlers_unblock_by_func (G_OBJECT (wid->priv->combo_entry),
+					   G_CALLBACK (new_combo_contents_changed_cb), wid);
 #endif
 }
 
@@ -270,16 +274,17 @@ mg_entry_combo_new (MgConf *conf, MgContext *context, MgContextNode *node)
 	/* create the combo itself */
 #ifdef USE_OLD_GTK_COMBO_WIDGET
 	entry = gtk_combo_new ();
+	g_signal_connect (G_OBJECT (GTK_COMBO (entry)->entry), "changed",
+			  G_CALLBACK (old_combo_contents_changed_cb), combo);
+	gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (entry)->entry), FALSE);
+#else
+	entry = gtk_combo_box_new ();
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (new_combo_contents_changed_cb), combo);
+#endif
 	mg_entry_shell_pack_entry (MG_ENTRY_SHELL (combo), entry);
 	gtk_widget_show (entry);
 	combo->priv->combo_entry = entry;
-	g_signal_connect (G_OBJECT (GTK_COMBO (entry)->entry), "changed",
-			  G_CALLBACK (combo_contents_changed_cb), combo);
-	gtk_editable_set_editable (GTK_EDITABLE (GTK_COMBO (entry)->entry), FALSE);
-#else
-	TO_IMPLEMENT;
-	entry = NULL;
-#endif
 
 	/* create the data model */
 	utility_combo_compute_model (combo->priv->ccore);
@@ -305,6 +310,124 @@ mg_entry_combo_new (MgConf *conf, MgContext *context, MgContextNode *node)
 	return GTK_WIDGET (obj);
 }
 
+#ifndef USE_OLD_GTK_COMBO_WIDGET
+static void cell_layout_data_func (GtkCellLayout *cell_layout, GtkCellRenderer *cell,
+				   GtkTreeModel *tree_model, GtkTreeIter *iter, MgEntryCombo *combo);
+
+/*
+ * Uses @model to first create the GtkTreeModel model and set the correct columns to the @view
+ * cell layout
+ */
+static void
+combo_make_model_and_columns (MgEntryCombo *combo)
+{
+	GtkListStore *tree_model;
+	gboolean has_layout = g_object_get_data (G_OBJECT (combo->priv->combo_entry), "mymodel") ? TRUE : FALSE;
+	
+	/* compute GtkTreeModel */
+	if (combo->priv->ccore->data_model_valid) {
+		/* data model : one column in which we store the data model's row number */
+		gint row_count = 0;
+		GtkTreeIter iter;
+		gint i;
+
+		tree_model = gtk_list_store_new (1, G_TYPE_INT);
+
+		/* storing each row number */
+		row_count = mg_resultset_get_nbtuples (combo->priv->ccore->resultset);
+		for (i = 0; i < row_count; i++) {
+			gtk_list_store_append (tree_model, &iter);
+			gtk_list_store_set (tree_model, &iter, 0, i, -1);
+		}
+
+		/* if NULL is possible, we add an empty line */
+		if (combo->priv->null_possible) {
+			gtk_list_store_prepend (tree_model, &iter);
+			gtk_list_store_set (tree_model, &iter, 0, -1, -1);
+			row_count++;
+		}
+	}
+	else {
+		/* data model : one column in which we store the error as a string */
+		GtkTreeIter iter;
+		gchar *str;
+		const GdaValue *value;
+
+		tree_model = gtk_list_store_new (1, G_TYPE_STRING);
+		gtk_list_store_append (tree_model, &iter);
+		value = gda_data_model_get_value_at (combo->priv->ccore->data_model, 0, 0);
+		str = gda_value_stringify (value);
+		gtk_list_store_set (tree_model, &iter, 0, str, -1);
+		g_free (str);
+	}
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo->priv->combo_entry), GTK_TREE_MODEL (tree_model));
+	g_object_set_data (G_OBJECT (combo->priv->combo_entry), "mymodel", tree_model);
+	g_object_unref (tree_model);
+
+	/* Layout */
+	if (has_layout) 
+		gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo->priv->combo_entry));
+	
+	if (combo->priv->ccore->data_model_valid) {
+		gint *mask, masksize;
+		
+		mask = combo->priv->ccore->mask;
+		masksize = combo->priv->ccore->masksize;
+		if (mask) {
+			GtkCellRenderer *renderer;
+			gint j;
+			MgField *field;
+			MgDataHandler *dh;
+
+			for (j=0; j<masksize; j++) {
+				field = mg_entity_get_field_by_index (MG_ENTITY (combo->priv->ccore->query),
+								      mask [j]);
+				g_assert (field);
+				dh = mg_server_data_type_get_handler (mg_field_get_data_type (field));
+				
+				renderer = gtk_cell_renderer_text_new ();
+				g_object_set_data (G_OBJECT (renderer), "data_handler", dh);
+				g_object_set_data (G_OBJECT (renderer), "colnum", GINT_TO_POINTER (mask [j]));
+				gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo->priv->combo_entry), renderer, TRUE);
+				gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (combo->priv->combo_entry), renderer,
+								    (GtkCellLayoutDataFunc) cell_layout_data_func, combo, NULL);
+			}
+		}
+	}
+	else {
+		GtkCellRenderer *renderer;
+
+		renderer = gtk_cell_renderer_text_new ();
+		gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo->priv->combo_entry), renderer, TRUE);
+		gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo->priv->combo_entry), renderer, 
+						"text", 0, NULL);
+	}
+
+}
+
+static void
+cell_layout_data_func (GtkCellLayout *cell_layout, GtkCellRenderer *cell,
+		       GtkTreeModel *tree_model, GtkTreeIter *iter, MgEntryCombo *combo)
+{
+	MgDataHandler *dh;
+	gint colnum, rownum;
+	const GdaValue *value;
+	gchar *str;
+	
+	dh = g_object_get_data (G_OBJECT (cell), "data_handler");
+	colnum = GPOINTER_TO_INT (g_object_get_data  (G_OBJECT (cell), "colnum"));
+	gtk_tree_model_get (tree_model, iter, 0, &rownum, -1);
+	if (rownum >= 0) {
+		value = gda_data_model_get_value_at (combo->priv->ccore->data_model, colnum, rownum);
+		str = mg_data_handler_get_str_from_value (dh, value);
+		g_object_set (G_OBJECT (cell), "text", str, NULL);
+		g_free (str);
+	}
+	else
+		g_object_set (G_OBJECT (cell), "text", "", NULL);
+}
+
+#endif
 
 /*
  * Displays the combo->priv->ccore->data_model model in the real combo widget, without emitting
@@ -332,10 +455,9 @@ display_combo_model (MgEntryCombo *combo)
 	}
 	g_list_free (strings);
 #else
-	/* TODO: use a widget which operates with something similar to GtkTreeIter where
-	 * it is possible to add / removed the 1st empty line without making all over again the
-	 * complete list of possible selectable items */
-	TO_IMPLEMENT;
+	real_combo_block_signals (combo);
+	combo_make_model_and_columns (combo);
+	real_combo_unblock_signals (combo);
 #endif
 }
 
@@ -343,14 +465,15 @@ display_combo_model (MgEntryCombo *combo)
 static void
 choose_auto_default_value (MgEntryCombo *combo)
 {
-#ifdef USE_OLD_GTK_COMBO_WIDGET
 	gint pos = 0;
 	if (combo->priv->null_possible)
 		pos ++;
-	set_selection_combo_model (combo, pos);
-	combo_contents_changed_cb (NULL, combo);
+	
+#ifdef USE_OLD_GTK_COMBO_WIDGET
+	old_set_selection_combo_model (combo, pos);
+	old_combo_contents_changed_cb (NULL, combo);
 #else
-	TO_IMPLEMENT;
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo->priv->combo_entry), pos);
 #endif
 }
 
@@ -461,7 +584,7 @@ mg_entry_combo_get_property (GObject              *object,
 
 #ifdef USE_OLD_GTK_COMBO_WIDGET
 static void
-combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo)
+old_combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo)
 {
 	gint selection;
 	
@@ -506,12 +629,65 @@ combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo)
 
 /* Forces the selection of an item in the items list, without emitting a single signal */
 static void
-set_selection_combo_model (MgEntryCombo *combo, gint pos)
+old_set_selection_combo_model (MgEntryCombo *combo, gint pos)
 {
 	real_combo_block_signals (combo);
 	gtk_list_select_item (GTK_LIST (GTK_COMBO (combo->priv->combo_entry)->list), pos);
 	real_combo_unblock_signals (combo);
 }
+#else
+static void
+new_combo_contents_changed_cb (GtkWidget *entry, MgEntryCombo *combo)
+{
+	gint selection;
+	
+	selection = gtk_combo_box_get_active (GTK_COMBO_BOX (entry));
+	if (selection < 0) {
+		g_warning ("GtkComboBox's selection < 0!");
+		return;
+	}
+
+	if ((selection == 0) && combo->priv->null_possible) /* Set to NULL? */
+		mg_entry_combo_set_values (combo, NULL);
+	else {
+		if (combo->priv->null_possible)
+			selection --; /* remove the "" entry count */
+
+		if (combo->priv->ccore->data_model_valid) {
+			combo->priv->null_forced = FALSE;
+			combo->priv->default_forced = FALSE;
+			combo->priv->data_valid = TRUE;
+			
+			/* updating the node's values */
+			if (combo->priv->ccore->data_model && combo->priv->ccore->data_model_valid &&
+			    (selection >= 0)) {
+				GSList *list;
+				list = combo->priv->ccore->nodes;
+				while (list) {
+					ComboNode *node = COMBO_NODE (list->data);
+					node->value = gda_data_model_get_value_at (combo->priv->ccore->data_model, 
+										   node->position, selection);
+					/* g_print ("%s(): Set Node value to %s\n", __FUNCTION__, */
+/* 						 node->value ? gda_value_stringify (node->value) : "Null"); */
+					list = g_slist_next (list);
+				}
+			}
+			
+			g_signal_emit_by_name (G_OBJECT (combo), "status_changed");
+			mg_entry_combo_emit_signal (combo);
+		}
+	}
+}
+
+
+/* Forces the selection of an item in the items list, without emitting a single signal */
+static void
+new_set_selection_combo_model (MgEntryCombo *combo, gint pos)
+{
+	real_combo_block_signals (combo);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo->priv->combo_entry), pos);
+	real_combo_unblock_signals (combo);
+}
 
 #endif
 
@@ -612,9 +788,9 @@ mg_entry_combo_set_values (MgEntryCombo *combo, GList *values)
 			combo->priv->null_forced = FALSE;
 			combo->priv->default_forced = FALSE;
 #ifdef USE_OLD_GTK_COMBO_WIDGET
-			set_selection_combo_model (combo, current_index);
+			old_set_selection_combo_model (combo, current_index);
 #else
-			TO_IMPLEMENT;
+			new_set_selection_combo_model (combo, current_index);
 #endif
 		}
 	}
@@ -624,9 +800,9 @@ mg_entry_combo_set_values (MgEntryCombo *combo, GList *values)
 		
 			/* the 1st entry in the real combo is already a "" entry, so select it. */
 #ifdef USE_OLD_GTK_COMBO_WIDGET
-			set_selection_combo_model (combo, 0);
+			old_set_selection_combo_model (combo, 0);
 #else
-			TO_IMPLEMENT;
+			new_set_selection_combo_model (combo, 0);
 #endif
 
 			/* adjusting the values */
@@ -822,7 +998,7 @@ static void
 mg_entry_combo_set_value (MgDataEntry *iface, const GdaValue *value)
 {
 	MgEntryCombo *combo;
-                                                                                                                                  
+
         g_return_if_fail (iface && IS_MG_ENTRY_COMBO (iface));
         combo = MG_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
@@ -837,11 +1013,11 @@ static GdaValue *
 mg_entry_combo_get_value (MgDataEntry *iface)
 {
         MgEntryCombo *combo;
-                                                                                                                                  
+	
         g_return_val_if_fail (iface && IS_MG_ENTRY_COMBO (iface), NULL);
         combo = MG_ENTRY_COMBO (iface);
         g_return_val_if_fail (combo->priv, NULL);
-                                                                                                                                  
+	
 	TO_IMPLEMENT;
 	
 	return NULL;
@@ -851,19 +1027,19 @@ static void
 mg_entry_combo_set_value_orig (MgDataEntry *iface, const GdaValue * value)
 {
         MgEntryCombo *combo;
-                                                                                                                                  
+	
         g_return_if_fail (iface && IS_MG_ENTRY_COMBO (iface));
         combo = MG_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
-                                                                                                                                  
+
 	TO_IMPLEMENT;
 }
-                                                                                                                                  
+
 static const GdaValue *
 mg_entry_combo_get_value_orig (MgDataEntry *iface)
 {
         MgEntryCombo *combo;
-                                                                                                                                  
+	
         g_return_val_if_fail (iface && IS_MG_ENTRY_COMBO (iface), NULL);
         combo = MG_ENTRY_COMBO (iface);
         g_return_val_if_fail (combo->priv, NULL);
@@ -877,11 +1053,11 @@ static void
 mg_entry_combo_set_value_default (MgDataEntry *iface, const GdaValue * value)
 {
         MgEntryCombo *combo;
-                                                                                                                                  
+	
         g_return_if_fail (iface && IS_MG_ENTRY_COMBO (iface));
         combo = MG_ENTRY_COMBO (iface);
         g_return_if_fail (combo->priv);
-                                                                                                                                  
+	
 	TO_IMPLEMENT;
 }
 
@@ -1053,11 +1229,11 @@ mg_entry_combo_get_attributes (MgDataEntry *iface)
 		/* is unchanged */
 		if (COMBO_NODE (list->data)->value_orig) {
 			orig_value_exists = TRUE;
-
+			
 			if (COMBO_NODE (list->data)->value && 
 			    (gda_value_get_type (COMBO_NODE (list->data)->value) == 
 			     gda_value_get_type (COMBO_NODE (list->data)->value_orig))) {
-				if (gda_value_get_type (COMBO_NODE (list->data)->value) == GDA_VALUE_TYPE_NULL)
+				if (gda_value_get_type (COMBO_NODE (list->data)->value) == GDA_VALUE_TYPE_NULL) 
 					changed = FALSE;
 				else {
 					if (gda_value_compare (COMBO_NODE (list->data)->value, 
@@ -1068,10 +1244,11 @@ mg_entry_combo_get_attributes (MgDataEntry *iface)
 			else
 				changed = TRUE;
 		}
-
-		if (changed || !orig_value_exists)
+		
+		if (changed || 
+		    (!orig_value_exists && !isnull))
 			isunchanged = FALSE;
-
+		
 		list = g_slist_next (list);
 	}
 
