@@ -27,6 +27,7 @@
 #include "mg-referer.h"
 #include "mg-entity.h"
 #include "mg-server-data-type.h"
+#include "mg-qfield.h"
 
 /* 
  * Main static functions 
@@ -59,10 +60,11 @@ static GObjectClass  *parent_class = NULL;
 /* signals */
 enum
 {
+	PARAM_CHANGED,
 	LAST_SIGNAL
 };
 
-static gint mg_context_signals[LAST_SIGNAL] = { };
+static gint mg_context_signals[LAST_SIGNAL] = { 0 };
 
 /* properties */
 enum
@@ -78,6 +80,8 @@ struct _MgContextPrivate
 	GHashTable *param_default_values;      /* key = param, value = new GdaValue */
 	GHashTable *param_default_aliases;     /* key = param, value = alias parameter */
 	GHashTable *aliases_default_param;     /* key = alias parameter, value = param */
+
+	GHashTable *param_repl;                /* key = parameter, value = parameter used instead because of similarity */
 };
 
 
@@ -123,6 +127,17 @@ mg_context_class_init (MgContextClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
+	mg_context_signals[PARAM_CHANGED] =
+		g_signal_new ("param_changed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (MgContextClass, param_changed),
+			      NULL, NULL,
+			      marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
+
+	class->param_changed = NULL;
+
 	object_class->dispose = mg_context_dispose;
 	object_class->finalize = mg_context_finalize;
 
@@ -149,30 +164,29 @@ mg_context_init (MgContext *mg_context)
 									NULL, (GDestroyNotify) gda_value_free);
 	mg_context->priv->param_default_aliases = g_hash_table_new (NULL, NULL);
 	mg_context->priv->aliases_default_param = g_hash_table_new (NULL, NULL);
+
+	mg_context->priv->param_repl = g_hash_table_new (NULL, NULL);
 }
 
+static void compute_context_nodes (MgContext *context);
+static void mg_context_real_add_param (MgContext *context, MgParameter *param);
 
 /**
  * mg_context_new
  * @conf: a #MgConf object
  * @params: a list of #MgParameter objects
  *
- * Creates a new #MgContext structure, and populates it with the list given as argument.
+ * Creates a new #MgContext object, and populates it with the list given as argument.
  * The list can then be freed as it gets copied. All the parameters in @params are referenced counted
  * and modified, so they should not be used anymore afterwards, and the @params list gets copied
  * (so it should be freed by the caller).
  *
- * Returns: a new #MgContext structure
+ * Returns: a new #MgContext object
  */
 GObject *
 mg_context_new (MgConf *conf, GSList *params)
 {
 	GObject *obj;
-	GHashTable *hash;       /* key = query referenced by a param, value= corresponding node */
-	GHashTable *node_hashs; /* key = node, value = corresponding replacements hash */
-
-	gboolean err = FALSE;
-	GSList *list;
 	MgContext *mg_context;
 
 	g_return_val_if_fail (conf && IS_MG_CONF (conf), NULL);
@@ -181,26 +195,308 @@ mg_context_new (MgConf *conf, GSList *params)
         mg_context = MG_CONTEXT (obj);
         mg_base_set_id (MG_BASE (mg_context), 0);
 
-	if (params)
-		mg_context->parameters = g_slist_copy (params);
-	mg_context->nodes = NULL;
+	/* add the parameters */
+	while (params) {
+		mg_context_real_add_param (mg_context, MG_PARAMETER (params->data));
+		params = g_slist_next (params);
+	}
+	compute_context_nodes (mg_context);
 
-	/* getting signals from the parameters */
-	list = mg_context->parameters;
-	while (list) {
-		g_signal_connect (G_OBJECT (list->data), "nullified",
-				  G_CALLBACK (nullified_param_cb), mg_context);
-		g_signal_connect (G_OBJECT (list->data), "changed",
-				  G_CALLBACK (changed_param_cb), mg_context);
+	return obj;
+}
 
-		g_object_ref (G_OBJECT (list->data));
-		list = g_slist_next (list);
+
+/**
+ * mg_context_new_copy
+ * @orig: a #MgContext to make a copy of
+ * @copy_params: TRUE to also make a copy of the #MgParameter objects within @orig
+ * @replacements: a place where to store the parameters and queries replacements, or %NULL
+ * 
+ * Copy constructor. @replacements is used and modified only if @copy_params is TRUE.
+ *
+ * Returns: a the new copy of @orig
+ */
+GObject *
+mg_context_new_copy (MgContext *orig, gboolean copy_params, GHashTable *replacements)
+{
+	GObject   *obj;
+	MgContext *mg_context;
+	MgConf *conf;
+	GSList *list;
+
+	g_return_val_if_fail (orig && IS_MG_CONTEXT (orig), NULL);
+
+	conf = mg_base_get_conf (MG_BASE (orig));
+	obj = g_object_new (MG_CONTEXT_TYPE, "conf", conf, NULL);
+	mg_context = MG_CONTEXT (obj);
+	mg_base_set_id (MG_BASE (mg_context), 0);
+
+	if (! copy_params) {
+		GSList *olist;
+		/* copy parameters */
+		if (orig->parameters)
+			mg_context->parameters = g_slist_copy (orig->parameters);
+		
+		olist = orig->parameters;
+		list = mg_context->parameters;
+		while (list) {
+			g_signal_connect (G_OBJECT (list->data), "nullified",
+					  G_CALLBACK (nullified_param_cb), mg_context);
+			g_signal_connect (G_OBJECT (list->data), "changed",
+					  G_CALLBACK (changed_param_cb), mg_context);
+			g_object_ref (G_OBJECT (list->data));
+
+			if (replacements)
+				g_hash_table_insert (replacements, olist->data, list->data);
+
+			list = g_slist_next (list);
+			olist = g_slist_next (olist);
+		}
+
+		/* copy nodes */
+		list = orig->nodes;
+		while (list) {
+			MgContextNode *node, *orig;
+			
+			orig = MG_CONTEXT_NODE (list->data);
+			node = g_new0 (MgContextNode, 1);
+			node->param = orig->param;
+			if (orig->query) {
+				node->query = orig->query;
+				g_object_ref (G_OBJECT (node->query));
+			}
+			if (orig->params)
+				node->params = g_slist_copy (orig->params);
+			
+			mg_context->nodes = g_slist_append (mg_context->nodes, node);
+			
+			if (replacements)
+				g_hash_table_insert (replacements, list->data, node);
+
+			list = g_slist_next (list);
+		}
+	}
+	else {
+		GHashTable *repl;
+		GSList *list;
+		MgParameter *param;
+
+		if (replacements)
+			repl = replacements;
+		else
+			repl = g_hash_table_new (NULL, NULL);
+		/* copy parameters */
+		if (orig->parameters) {
+			list = orig->parameters;
+			while (list) {
+				param = MG_PARAMETER (mg_parameter_new_copy (MG_PARAMETER (list->data)));
+				g_hash_table_insert (repl, list->data, param);
+				mg_context->parameters = g_slist_append (mg_context->parameters, param);
+
+				g_signal_connect (G_OBJECT (param), "nullified",
+						  G_CALLBACK (nullified_param_cb), mg_context);
+				g_signal_connect (G_OBJECT (param), "changed",
+						  G_CALLBACK (changed_param_cb), mg_context);
+
+				list = g_slist_next (list);
+			}
+		}
+		
+		/* finish nodes copy */
+		list = orig->nodes;
+		while (list) {
+			MgContextNode *node, *orig;
+			
+			orig = MG_CONTEXT_NODE (list->data);
+			node = g_new0 (MgContextNode, 1);
+
+			node->param = g_hash_table_lookup (repl, orig->param);
+			
+			if (orig->params) {
+				list = orig->params;
+				while (list) {
+					node->params = g_slist_append (node->params, g_hash_table_lookup (repl, list->data));
+					list = g_slist_next (list);
+				}
+			}
+
+			if (orig->query) 
+				node->query = MG_QUERY (mg_query_new_copy (orig->query, repl));
+			
+			mg_context->nodes = g_slist_append (mg_context->nodes, node);
+			g_hash_table_insert (repl, orig, node);
+
+			list = g_slist_next (list);
+		}
+
+		/* make the replacements be taken into account */
+		list = mg_context->parameters;
+		while (list) {
+			mg_referer_replace_refs (MG_REFERER (list->data), repl);
+			list = g_slist_next (list);
+		}
+		
+		if (!replacements)
+			g_hash_table_destroy (repl);
 	}
 
-	/* creating the MgContextNode entries */
+	return obj;
+}
+
+static void
+nullified_param_cb (MgParameter *param, MgContext *context)
+{
+	g_assert (g_slist_find (context->parameters, param));
+	context->parameters = g_slist_remove (context->parameters, param);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
+					      G_CALLBACK (nullified_param_cb), context);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
+					      G_CALLBACK (changed_param_cb), context);
+
+	if (context->nodes) {
+		MgContextNode *node;
+		gboolean finished = FALSE;
+		GSList *list = context->nodes;
+
+		while (list && !finished) {
+			node = MG_CONTEXT_NODE (list->data);
+			if (node->params && g_slist_find (node->params, param)) {
+				node->params = g_slist_remove (node->params, param);
+				if (!node->params) {
+					g_object_unref (node->query);
+					context->nodes = g_slist_remove (context->nodes, node);
+					g_free (node);
+				}
+				finished = TRUE;
+			}
+			list = g_slist_next (list);
+		}
+	}
+	g_object_unref (G_OBJECT (param));
+}
+
+static void
+changed_param_cb (MgParameter *param, MgContext *context)
+{
+	/* signal the parameter change */
+	mg_base_changed (MG_BASE (context));
+#ifdef debug_signal
+	g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (context), mg_context_signals[PARAM_CHANGED], 0, param);
+#ifdef debug_signal
+	g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
+#endif
+}
+
+static void
+mg_context_dispose (GObject *object)
+{
+	MgContext *mg_context;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (IS_MG_CONTEXT (object));
+
+	mg_context = MG_CONTEXT (object);
+	if (mg_context->priv) {
+		mg_base_nullify_check (MG_BASE (object));
+	}
+	
+	/* free the nodes if there are some */
+	while (mg_context->nodes)
+		mg_context_free_node (mg_context, MG_CONTEXT_NODE (mg_context->nodes->data));
+
+	/* free the parameters list */
+	while (mg_context->parameters)
+		nullified_param_cb (MG_PARAMETER (mg_context->parameters->data), mg_context);
+
+
+	/* parent class */
+	parent_class->dispose (object);
+}
+
+static void foreach_finalize_alias (MgParameter *param, MgParameter *alias, MgContext *context);
+static void
+mg_context_finalize (GObject *object)
+{
+	MgContext *mg_context;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (IS_MG_CONTEXT (object));
+
+	mg_context = MG_CONTEXT (object);
+	if (mg_context->priv) {
+		g_hash_table_destroy (mg_context->priv->param_default_values);
+		g_hash_table_foreach (mg_context->priv->param_default_aliases,
+				      (GHFunc) foreach_finalize_alias, mg_context);
+		g_hash_table_destroy (mg_context->priv->param_default_aliases);
+		g_hash_table_destroy (mg_context->priv->aliases_default_param);
+		g_hash_table_destroy (mg_context->priv->param_repl);
+
+		g_free (mg_context->priv);
+		mg_context->priv = NULL;
+	}
+
+	/* parent class */
+	parent_class->finalize (object);
+}
+
+
+static void 
+mg_context_set_property (GObject              *object,
+			guint                 param_id,
+			const GValue         *value,
+			GParamSpec           *pspec)
+{
+	gpointer ptr;
+	MgContext *mg_context;
+
+	mg_context = MG_CONTEXT (object);
+	if (mg_context->priv) {
+		switch (param_id) {
+		case PROP:
+			ptr = g_value_get_pointer (value);
+			break;
+		}
+	}
+}
+
+static void
+mg_context_get_property (GObject              *object,
+			  guint                 param_id,
+			  GValue               *value,
+			  GParamSpec           *pspec)
+{
+	MgContext *mg_context;
+	mg_context = MG_CONTEXT (object);
+	
+	if (mg_context->priv) {
+		switch (param_id) {
+		case PROP:
+			break;
+		}	
+	}
+}
+
+/*
+ * Computes context->nodes, and if some nodes already exist, they are previously discarded
+ */
+static void 
+compute_context_nodes (MgContext *context)
+{
+	gboolean err = FALSE;
+	GSList *list;
+	GHashTable *hash;       /* key = query referenced by a param, value= corresponding node */
+	GHashTable *node_hashs; /* key = node, value = corresponding replacements hash */
+
+	/* free the nodes if there are some */
+	while (context->nodes)
+		mg_context_free_node (context, MG_CONTEXT_NODE (context->nodes->data));
+
 	hash = g_hash_table_new (NULL, NULL);
 	node_hashs = g_hash_table_new (NULL, NULL);
-	list = mg_context->parameters;
+	list = context->parameters;
+
 	while (list && !err) {
 		MgQfield *ref_field;
 		
@@ -224,7 +520,7 @@ mg_context_new (MgConf *conf, GSList *params)
 					g_hash_table_insert (hash, ent, node);
 					g_hash_table_insert (node_hashs, node, repl);
 
-					mg_context->nodes = g_slist_append (mg_context->nodes, node);
+					context->nodes = g_slist_append (context->nodes, node);
 				}
 				else 
 					enode->params = g_slist_append (enode->params, list->data);
@@ -233,17 +529,17 @@ mg_context_new (MgConf *conf, GSList *params)
 		else {
 			/* the parameter is free fill */
 			MgContextNode *node;
-
+			
 			node = g_new0 (MgContextNode, 1);
 			node->param = list->data;
-			mg_context->nodes = g_slist_append (mg_context->nodes, node);
+			context->nodes = g_slist_append (context->nodes, node);
 		}
 
 		list = g_slist_next (list);
 	}
 
 	/* taking into account all the replacements */
-	list = mg_context->nodes;
+	list = context->nodes;
 	while (list) {
 		GHashTable *node_repl, *dep_repl;
 		MgQfield *ref_field;
@@ -311,7 +607,7 @@ mg_context_new (MgConf *conf, GSList *params)
 		
 		if (node->query) {
 			/* replacements for the query */
-			GSList *sublist = mg_context->nodes;
+			GSList *sublist = context->nodes;
 
 			while (sublist) {
 				if (MG_CONTEXT_NODE (sublist->data)->query)
@@ -324,7 +620,7 @@ mg_context_new (MgConf *conf, GSList *params)
 	}
 
 	/* creating params_pos_in_query in each node */
-	list = mg_context->nodes;
+	list = context->nodes;
 	while (list) {
 		GSList *params;
 		GHashTable *hash_pos;
@@ -338,7 +634,11 @@ mg_context_new (MgConf *conf, GSList *params)
 		while (params) {
 			field = mg_parameter_get_source_field (MG_PARAMETER (params->data));
 			pos = mg_entity_get_field_index (MG_ENTITY (MG_CONTEXT_NODE (list->data)->query), MG_FIELD (field));
-			g_assert (pos>=0);
+			if (pos < 0) {
+				err = TRUE;
+				g_warning ("Can't find position of field %p into query %p for parameter %p\n",
+					   field, MG_CONTEXT_NODE (list->data)->query, params->data);
+			}
 
 			g_hash_table_insert (hash_pos, params->data, GINT_TO_POINTER (pos));
 			params = g_slist_next (params);
@@ -348,7 +648,7 @@ mg_context_new (MgConf *conf, GSList *params)
 	}
 
 	/* memory de-allocations */
-	list = mg_context->nodes;
+	list = context->nodes;
 	while (list) {
 		GHashTable *node_repl;
 		node_repl = g_hash_table_lookup (node_hashs, list->data);
@@ -361,193 +661,98 @@ mg_context_new (MgConf *conf, GSList *params)
 	g_hash_table_destroy (hash);
 	g_hash_table_destroy (node_hashs);
 
-	return obj;
+#ifdef debug_NO
+	mg_context_dump (context, 0);
+#endif
 }
 
-
 /**
- * mg_context_new_copy
- * @orig: a #MgContext to make a copy of
- * 
- * Copy constructor
+ * mg_context_add_param
+ * @context: a #MgContext object
+ * @param: a #MgParameter object
  *
- * Returns: a the new copy of @orig
+ * Adds @param to the list of parameters managed within @context.
+ * <b>WARNING:</b> the context may decide not to use the @param parameter, but to
+ * modify another parameter already present within the context. The publicly available
+ * lists from the @context object may also be changed in the process.
  */
-GObject *
-mg_context_new_copy (MgContext *orig)
+void
+mg_context_add_param (MgContext *context, MgParameter *param)
 {
-	GObject   *obj;
-	MgContext *mg_context;
-	MgConf *conf;
-	GSList *list;
+	g_return_if_fail (context && IS_MG_CONTEXT (context));
+	g_return_if_fail (param && IS_MG_PARAMETER (param));
 
-	g_return_val_if_fail (orig && IS_MG_CONTEXT (orig), NULL);
+	mg_context_real_add_param (context, param);
+	compute_context_nodes (context);
+}
 
-	conf = mg_base_get_conf (MG_BASE (orig));
-	obj = g_object_new (MG_CONTEXT_TYPE, "conf", conf, NULL);
-	mg_context = MG_CONTEXT (obj);
-	mg_base_set_id (MG_BASE (mg_context), 0);
+static void
+mg_context_real_add_param (MgContext *context, MgParameter *param)
+{
+	GSList *params;
+	MgParameter *similar = NULL;
+	GSList *param_dest_fields;
 
-	/* copy parameters */
-	if (orig->parameters)
-		mg_context->parameters = g_slist_copy (orig->parameters);
+	if (g_slist_find (context->parameters, param))
+		return;
 
-	list = mg_context->parameters;
-	while (list) {
-		g_signal_connect (G_OBJECT (list->data), "nullified",
-				  G_CALLBACK (nullified_param_cb), mg_context);
-		g_signal_connect (G_OBJECT (list->data), "changed",
-				  G_CALLBACK (changed_param_cb), mg_context);
-		g_object_ref (G_OBJECT (list->data));
-		list = g_slist_next (list);
-	}
+	/* try to find a similar parameter in the context->parameters */
+	param_dest_fields = mg_parameter_get_dest_fields (param);
+	params = context->parameters;
+	while (params && !similar) {
+		GSList *list1;
+		list1 = mg_parameter_get_dest_fields (MG_PARAMETER (params->data));
 
-	/* copy nodes */
-	list = orig->nodes;
-	while (list) {
-		MgContextNode *node, *orig;
-
-		orig = MG_CONTEXT_NODE (list->data);
-		node = g_new0 (MgContextNode, 1);
-		node->param = orig->param;
-		if (orig->query) {
-			node->query = orig->query;
-			g_object_ref (G_OBJECT (node->query));
+		while (list1 && !similar) {
+			if (g_slist_find (param_dest_fields, list1->data))
+				similar = MG_PARAMETER (params->data);
+			list1 = g_slist_next (list1);
 		}
-		if (orig->params)
-			node->params = g_slist_copy (orig->params);
-		
-		mg_context->nodes = g_slist_append (mg_context->nodes, node);
-		list = g_slist_next (list);
-	}
-
-	return obj;	
-}
-
-static void
-nullified_param_cb (MgParameter *param, MgContext *context)
-{
-	g_assert (g_slist_find (context->parameters, param));
-	context->parameters = g_slist_remove (context->parameters, param);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
-					      G_CALLBACK (nullified_param_cb), context);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (param),
-					      G_CALLBACK (changed_param_cb), context);
-
-	if (context->nodes) {
-		MgContextNode *node;
-		gboolean finished = FALSE;
-		GSList *list = context->nodes;
-
-		while (list && !finished) {
-			node = MG_CONTEXT_NODE (list->data);
-			if (node->params && g_slist_find (node->params, param)) {
-				node->params = g_slist_remove (node->params, param);
-				if (!node->params) {
-					g_object_unref (node->query);
-					context->nodes = g_slist_remove (context->nodes, node);
-					g_free (node);
-				}
-				finished = TRUE;
-			}
-			list = g_slist_next (list);
-		}
-	}
-	g_object_unref (G_OBJECT (param));
-}
-
-static void
-changed_param_cb (MgParameter *param, MgContext *context)
-{
-	/* signal the parameter change */
-	mg_base_changed (MG_BASE (context));
-}
-
-static void
-mg_context_dispose (GObject *object)
-{
-	MgContext *mg_context;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_MG_CONTEXT (object));
-
-	mg_context = MG_CONTEXT (object);
-	if (mg_context->priv) {
-		mg_base_nullify_check (MG_BASE (object));
+		params = g_slist_next (params);
 	}
 	
-	/* free the nodes if there are some */
-	while (mg_context->nodes)
-		mg_context_free_node (mg_context, MG_CONTEXT_NODE (mg_context->nodes->data));
-
-	/* free the parameters list */
-	while (mg_context->parameters)
-		nullified_param_cb (MG_PARAMETER (mg_context->parameters->data), mg_context);
-
-
-	/* parent class */
-	parent_class->dispose (object);
-}
-
-static void foreach_finalize_alias (MgParameter *param, MgParameter *alias, MgContext *context);
-static void
-mg_context_finalize (GObject *object)
-{
-	MgContext *mg_context;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_MG_CONTEXT (object));
-
-	mg_context = MG_CONTEXT (object);
-	if (mg_context->priv) {
-		g_hash_table_destroy (mg_context->priv->param_default_values);
-		g_hash_table_foreach (mg_context->priv->param_default_aliases,
-				      (GHFunc) foreach_finalize_alias, mg_context);
-		g_hash_table_destroy (mg_context->priv->param_default_aliases);
-		g_hash_table_destroy (mg_context->priv->aliases_default_param);
-
-		g_free (mg_context->priv);
-		mg_context->priv = NULL;
-	}
-
-	/* parent class */
-	parent_class->finalize (object);
-}
-
-
-static void 
-mg_context_set_property (GObject              *object,
-			guint                 param_id,
-			const GValue         *value,
-			GParamSpec           *pspec)
-{
-	gpointer ptr;
-	MgContext *mg_context;
-
-	mg_context = MG_CONTEXT (object);
-	if (mg_context->priv) {
-		switch (param_id) {
-		case PROP:
-			ptr = g_value_get_pointer (value);
-			break;
+	if (similar) {
+#ifdef debug_NO
+		g_print ("Param %p and %p are similar, keeping %p only\n", similar, param, similar);
+#endif
+		g_hash_table_insert (context->priv->param_repl, param, similar);
+		/* don't use @param, but instead use similar and update the list of fields
+		   similar is for */
+		while (param_dest_fields) {
+			mg_parameter_add_dest_field (similar, MG_QFIELD (param_dest_fields->data));
+			param_dest_fields = g_slist_next (param_dest_fields);
 		}
 	}
+	else {
+		/* really add @param to the context */
+		context->parameters = g_slist_append (context->parameters, param);
+		g_signal_connect (G_OBJECT (param), "nullified",
+				  G_CALLBACK (nullified_param_cb), context);
+		g_signal_connect (G_OBJECT (param), "changed",
+				  G_CALLBACK (changed_param_cb), context);
+
+		g_object_ref (G_OBJECT (param));
+		mg_parameter_replace_ref (param, context->priv->param_repl);
+	}
 }
 
-static void
-mg_context_get_property (GObject              *object,
-			  guint                 param_id,
-			  GValue               *value,
-			  GParamSpec           *pspec)
+/**
+ * mg_context_merge_context_params
+ * @context: a #MgContext object
+ * @context_to_merge: a #MgContext object
+ *
+ * Add to @context all the parameters of @context_to_merge.
+ */
+void
+mg_context_merge_context_params (MgContext *context, MgContext *context_to_merge)
 {
-	MgContext *mg_context;
-	mg_context = MG_CONTEXT (object);
-	
-	if (mg_context->priv) {
-		switch (param_id) {
-		case PROP:
-			break;
-		}	
+	GSList *params = context_to_merge->parameters;
+	g_return_if_fail (context && IS_MG_CONTEXT (context));
+	g_return_if_fail (context_to_merge && IS_MG_CONTEXT (context_to_merge));
+
+	while (params) {
+		mg_context_add_param (context, MG_PARAMETER (params->data));
+		params = g_slist_next (params);
 	}
 }
 
@@ -563,7 +768,131 @@ mg_context_get_property (GObject              *object,
 gboolean
 mg_context_is_coherent (MgContext *context, GError **error)
 {
-	/* TODO */
+	GSList *list;
+	gint current_pos = 0;
+	/* To test:
+	   - For each parameter, check that the value provider is a field of the same type and is visible
+	   - For each parameter, check that the value provider belongs to a SELECT query
+	   - For each parameter, check that all the dependancies parameter are also in the context
+	   - the nodes settings are correct
+	*/
+	list = context->parameters;
+	while (list) {
+		MgParameter *param = MG_PARAMETER (list->data);
+		MgQfield *value_prov;
+		MgContextNode *node;
+		GSList *depend;
+		
+		node = mg_context_find_node_for_param (context, param);
+		if (!node) {
+			g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_NO_NODE_ERROR,
+				     _("Value provider for parameter '%s' is not listed in any context node "
+				       "(internal error)"),
+				     mg_base_get_name (MG_BASE (param)));
+			return FALSE;
+		}
+
+		/*
+		 * testing the value providers
+		 */
+		value_prov = mg_parameter_get_source_field (param);
+		if (value_prov) {
+			MgQuery *query;
+
+			/* type of object referenced by the "value_prov" property */
+			if (! IS_MG_QFIELD (value_prov)) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_VALUE_PROV_OBJ_TYPE_ERROR,
+					     _("Value provider for parameter '%s' is not a query field"),
+					     mg_base_get_name (MG_BASE (param)));
+				return FALSE;
+			}
+			
+			/* data type of value_prov */
+			if (mg_parameter_get_data_type (param) != mg_field_get_data_type (MG_FIELD (value_prov))) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_VALUE_PROV_DATA_TYPE_ERROR,
+					     _("Data types for parameter '%s' and value provider differ "
+					       "(respectively %s and %s)"),
+					     mg_base_get_name (MG_BASE (param)),
+					     mg_base_get_name (MG_BASE (mg_parameter_get_data_type (param))),
+					     mg_base_get_name (MG_BASE (mg_field_get_data_type (MG_FIELD (value_prov)))));
+				return FALSE;
+			}
+
+			/* value_prov is visible */
+			if (!mg_qfield_is_visible (MG_QFIELD (value_prov))) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_VALUE_PROV_INVISIBLE_ERROR,
+					     _("Value provider for parameter '%s' is not visible "
+					       "(won't appear in the result set when query is executed)"),
+					     mg_base_get_name (MG_BASE (param)));
+				return FALSE;
+			}
+
+			/* type of query to which value_prov belongs */
+			query = MG_QUERY (mg_field_get_entity (MG_FIELD (value_prov)));
+			if (! mg_query_is_select_query (query)) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_VALUE_PROV_QUERY_TYPE_ERROR,
+					     _("Value provider for parameter '%s' does not belong to a SELECT query"),
+					     mg_base_get_name (MG_BASE (param)));
+				return FALSE;
+			}
+
+			/* MgContextNode for this param is correct */
+			if (node->param ||
+			    (node->query != query) ||
+			    (GPOINTER_TO_INT (g_hash_table_lookup (node->params_pos_in_query, param)) < 0)) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_NODE_OUTDATED_ERROR,
+					     _("Parameter '%s' has changed since insertion in the context"),
+					     mg_base_get_name (MG_BASE (param)));
+				return FALSE;
+			}
+		}
+		else {
+			/* MgContextNode for this param is correct */	
+			if ((node->param != param) ||
+			    node->query ||
+			    node->params) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_NODE_OUTDATED_ERROR,
+					     _("Parameter '%s' has changed since insertion in the context"),
+					     mg_base_get_name (MG_BASE (param)));
+				return FALSE;
+			}
+		}
+
+		/*
+		 * testing the parameter dependencies which must be within the context and before
+		 * param in the parameters list
+		 */
+		depend = mg_parameter_get_dependencies (param);
+		while (depend) {
+			gint i = g_slist_index (context->parameters, depend->data);
+
+			/* dependency param in context */
+			if (i < 0) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_DEPENDENCY_NOT_IN_CONTEXT_ERROR,
+					     _("Parameter '%s' has a dependency parameter ('%s') which is "
+					       "not handled in the context"),
+					     mg_base_get_name (MG_BASE (param)),
+					     mg_base_get_name (MG_BASE (depend->data)));
+				return FALSE;
+			}
+			
+			/* dependency param before tested param in the parameters list */
+			if (i >= current_pos) {
+				g_set_error (error, MG_CONTEXT_ERROR, MG_CONTEXT_DEPENDENCY_POSITION_ERROR,
+					     _("Parameter '%s' has a dependency parameter ('%s') which is "
+					       "after itself the context"),
+					     mg_base_get_name (MG_BASE (param)),
+					     mg_base_get_name (MG_BASE (depend->data)));
+				return FALSE;
+			}
+
+			depend = g_slist_next (depend);
+		}
+
+		list = g_slist_next (list);
+		current_pos++;
+	}
+	
 	return TRUE;
 }
 
@@ -710,6 +1039,8 @@ mg_context_find_node_for_param (MgContext *context, MgParameter *param)
 
 	g_return_val_if_fail (context && IS_MG_CONTEXT (context), NULL);
 	g_return_val_if_fail (context->priv, NULL);
+	g_return_val_if_fail (param && IS_MG_PARAMETER (param), NULL);
+	g_return_val_if_fail (g_slist_find (context->parameters, param), NULL);
 
 	list = context->nodes;
 	while (list && !retval) {
