@@ -54,7 +54,6 @@ static gboolean    mg_database_load_from_xml (MgXmlStorage *iface, xmlNodePtr no
 
 static void mg_database_add_table (MgDatabase *mgdb, MgDbTable *table, gint pos);
 static void mg_database_add_sequence (MgDatabase *mgdb, MgDbSequence *seq, gint pos);
-static void mg_database_add_constraint (MgDatabase *mgdb, MgDbConstraint *cstr);
 
 static void nullified_table_cb (MgDbTable *table, MgDatabase *mgdb);
 static void nullified_sequence_cb (MgDbSequence *seq, MgDatabase *mgdb);
@@ -680,6 +679,7 @@ mg_database_load_from_xml_tables (MgXmlStorage *iface, xmlNodePtr node, GError *
 	return allok;
 }
 
+static void mg_database_add_constraint_real (MgDatabase *mgdb, MgDbConstraint *cstr, gboolean force_user_constraint);
 static gboolean
 mg_database_load_from_xml_constraints (MgXmlStorage *iface, xmlNodePtr node, GError **error)
 {
@@ -693,7 +693,7 @@ mg_database_load_from_xml_constraints (MgXmlStorage *iface, xmlNodePtr node, GEr
 				
 				cstr = MG_DB_CONSTRAINT (mg_db_constraint_new_with_db (MG_DATABASE (iface)));
 				allok = mg_xml_storage_load_from_xml (MG_XML_STORAGE (cstr), subnode, error);
-				mg_database_add_constraint (MG_DATABASE (iface), cstr);
+				mg_database_add_constraint_real (MG_DATABASE (iface), cstr, FALSE);
 				g_object_unref (G_OBJECT (cstr));
 			}
 			else {
@@ -771,11 +771,27 @@ mg_database_add_sequence (MgDatabase *mgdb, MgDbSequence *seq, gint pos)
 #endif
 }
 
-static void
+/**
+ * mg_database_add_constraint
+ * @mgdb: a #MgDatabase object
+ * @cstr: a #MgDbConstraint
+ *
+ * Add the @cstr constraint to the database. The @cstr constraint is a user-defined constraint
+ * (which is not part of the database structure itself).
+ */
+void
 mg_database_add_constraint (MgDatabase *mgdb, MgDbConstraint *cstr)
+{
+	mg_database_add_constraint_real (mgdb, cstr, TRUE);
+}
+
+static void
+mg_database_add_constraint_real (MgDatabase *mgdb, MgDbConstraint *cstr, gboolean force_user_constraint)
 {
 	MgDbConstraint *ptr = NULL;
 
+	g_return_if_fail (mgdb && IS_MG_DATABASE (mgdb));
+	g_return_if_fail (mgdb->priv);
 	g_return_if_fail (cstr);
 
 	/* Try to activate the constraints here */
@@ -801,6 +817,10 @@ mg_database_add_constraint (MgDatabase *mgdb, MgDbConstraint *cstr)
 		GSList *list;
 		MgDbTable *table;
 
+		/* user defined constraint */
+		if (force_user_constraint)
+			g_object_set (G_OBJECT (cstr), "user_constraint", TRUE, NULL);
+
 		/* add @cstr to the list of constraints */
 		mgdb->priv->constraints = g_slist_append (mgdb->priv->constraints, cstr);
 		g_object_ref (G_OBJECT (cstr));
@@ -1103,7 +1123,8 @@ database_tables_update_list (MgDatabase *mgdb, GError **error)
 			GSList *list = constraints;
 
 			while (list) {
-				mg_database_add_constraint (mgdb, MG_DB_CONSTRAINT (list->data));
+				mg_database_add_constraint_real (mgdb, MG_DB_CONSTRAINT (list->data), FALSE);
+				g_object_set (G_OBJECT (list->data), "user_constraint", FALSE, NULL);
 				g_object_unref (G_OBJECT (list->data));
 				list = g_slist_next (list);
 			}
@@ -1597,12 +1618,13 @@ mg_database_get_all_fk_constraints (MgDatabase *mgdb)
 /**
  * mg_database_get_tables_fk_constraints
  * @mgdb: a #MgDatabase object
- * @table1: a #MgDbTable
- * @table2: a #MgDbTable
+ * @table1: a #MgDbTable, or %NULL
+ * @table2: a #MgDbTable, or %NULL
  * @table1_has_fk: TRUE if the returned constraints are the one for which @table1 contains the foreign key
  *
- * Get a list of all the constraints applied to the database which represent a foreign constrains, between
- * @table1 and @table2.
+ * Get a list of all the constraints applied to the database which represent a foreign key constrains, between
+ * @table1 and @table2. If @table1 or @table2 are %NULL, then the returned foreign key constraints are the ones
+ * between any table and @table1 or @table2.
  *
  * Constraints are represented as #MgDbConstraint objects.
  *
@@ -1617,27 +1639,50 @@ mg_database_get_tables_fk_constraints (MgDatabase *mgdb, MgDbTable *table1, MgDb
 
 	g_return_val_if_fail (mgdb && IS_MG_DATABASE (mgdb), NULL);
 	g_return_val_if_fail (mgdb->priv, NULL);
-	g_return_val_if_fail (table1 && IS_MG_DB_TABLE (table1), NULL);
-	g_return_val_if_fail (table2 && IS_MG_DB_TABLE (table2), NULL);
-	if (table1 == table2)
+	if (table1)
+		g_return_val_if_fail (IS_MG_DB_TABLE (table1), NULL);
+	if (table2)
+		g_return_val_if_fail (IS_MG_DB_TABLE (table2), NULL);
+	if (!table1 && !table2)
 		return NULL;
 
-	list = g_hash_table_lookup (mgdb->priv->constraints_hash, table1);
-	while (list) {
-		fkptr = MG_DB_CONSTRAINT (list->data);
-		if ((mg_db_constraint_get_constraint_type (fkptr) == CONSTRAINT_FOREIGN_KEY) &&
-		    (mg_db_constraint_fkey_get_ref_table (fkptr) == MG_DB_TABLE (table2)))
-			retval = g_slist_append (retval, fkptr);
-		list = g_slist_next (list);
+	if (table1_has_fk) {
+		if (table1) {
+			/* we want the FK constraints where @table1 is the FK table */
+			list = g_hash_table_lookup (mgdb->priv->constraints_hash, table1);
+			while (list) {
+				fkptr = MG_DB_CONSTRAINT (list->data);
+				if ((mg_db_constraint_get_constraint_type (fkptr) == CONSTRAINT_FOREIGN_KEY) &&
+				    (!table2 || (mg_db_constraint_fkey_get_ref_table (fkptr) ==  (MgDbTable *) table2)))
+					retval = g_slist_append (retval, fkptr);
+				list = g_slist_next (list);
+			}
+		}
+		else {
+			/* we want all the FK constraints where @table2 is the ref_pk table */
+			list = mgdb->priv->constraints;
+			while (list) {
+				fkptr = MG_DB_CONSTRAINT (list->data);
+				if ((mg_db_constraint_get_constraint_type (fkptr) == CONSTRAINT_FOREIGN_KEY) &&
+				    (mg_db_constraint_fkey_get_ref_table (fkptr) ==  (MgDbTable *) table2))
+					retval = g_slist_append (retval, fkptr);
+				list = g_slist_next (list);
+			}
+		}
 	}
-
-	if (!table1_has_fk) {
-		list = g_hash_table_lookup (mgdb->priv->constraints_hash, table2);
+	else {
+		list = mgdb->priv->constraints;
 		while (list) {
 			fkptr = MG_DB_CONSTRAINT (list->data);
-			if ((mg_db_constraint_get_constraint_type (fkptr) == CONSTRAINT_FOREIGN_KEY) &&
-			    (mg_db_constraint_fkey_get_ref_table (fkptr) == MG_DB_TABLE (table1)))
-				retval = g_slist_append (retval, fkptr);
+			if (mg_db_constraint_get_constraint_type (fkptr) == CONSTRAINT_FOREIGN_KEY) {
+				MgDbTable *fk_table = mg_db_constraint_get_table (fkptr);
+				MgDbTable *ref_pk_table = mg_db_constraint_fkey_get_ref_table (fkptr);
+				
+				if (((!table1 || (fk_table == table1)) && (!table2 || (ref_pk_table == table2))) ||
+				    ((!table1 || (ref_pk_table == table1)) && (!table2 || (fk_table == table2))) )
+					retval = g_slist_append (retval, fkptr);
+			}
+				
 			list = g_slist_next (list);
 		}
 	}
