@@ -310,6 +310,8 @@ mg_db_table_dispose (GObject *object)
 
 		mg_base_nullify_check (MG_BASE (object));
 
+		mg_db_table_set_database (mg_db_table, NULL);
+
 		if (mg_db_table->priv->fields_hash) {
 			g_hash_table_destroy (mg_db_table->priv->fields_hash);
 			mg_db_table->priv->fields_hash = NULL;
@@ -699,7 +701,7 @@ mg_db_table_update_dbms_data (MgDbTable *table, GError **error)
 		
 		/* NOT NULL constraint */
 		value = gda_data_model_get_value_at (rs, 4, now);
-		if (value && !gda_value_is_null (value) && !gda_value_get_boolean (value)) {
+		if (value && !gda_value_is_null (value) && gda_value_get_boolean (value)) {
 			MgDbConstraint *cstr = MG_DB_CONSTRAINT (mg_db_constraint_new (table, CONSTRAINT_NOT_NULL));
 			mg_db_constraint_not_null_set_field (cstr, field);
 			constraints = g_slist_append (constraints, cstr);
@@ -937,16 +939,19 @@ mg_db_table_get_field_by_name (MgEntity *iface, const gchar *name)
 {
 	MgField *field = NULL;
 	GSList *list;
+	gchar *lcname = g_utf8_strdown (name, -1);
 
 	g_return_val_if_fail (iface && IS_MG_DB_TABLE (iface), NULL);
 	g_return_val_if_fail (MG_DB_TABLE (iface)->priv, NULL);
 
 	list = MG_DB_TABLE (iface)->priv->fields;
 	while (list && !field) {
-		if (!strcmp (mg_field_get_name (MG_FIELD (list->data)), name))
+		if (!strcmp (mg_field_get_name (MG_FIELD (list->data)), lcname) ||
+		    !strcmp (mg_field_get_name (MG_FIELD (list->data)), name))
 			field = MG_FIELD (list->data);
 		list = g_slist_next (list);
 	}
+	g_free (lcname);
 
 	return field;
 }
