@@ -1,6 +1,6 @@
 /* mg-enums.h
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2004 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,22 +22,43 @@
 #define __MG_ENUMS__
 
 typedef enum   _MgActionMode  MgActionMode;
+typedef enum   _MgAction      MgAction;
 
 /* enum for the different modes of action */
 enum _MgActionMode
 {
 	/* navigation modes */
-	MG_ACTION_NAVIGATION_ARROWS  = 1 << 0,
-	MG_ACTION_NAVIGATION_SCROLL  = 1 << 1,
+	MG_ACTION_NAVIGATION_ARROWS       = 1 << 0,
+	MG_ACTION_NAVIGATION_SCROLL       = 1 << 1,
 
 	/* modifications */
-	MG_ACTION_MODIF_AUTO_COMMIT  = 1 << 2,
-	MG_ACTION_ASK_CONFIRM_UPDATE = 1 << 3,
-	MG_ACTION_ASK_CONFIRM_DELETE = 1 << 4,
-	MG_ACTION_ASK_CONFIRM_INSERT = 1 << 5,
+	MG_ACTION_MODIF_AUTO_COMMIT       = 1 << 2,
+	MG_ACTION_MODIF_COMMIT_IMMEDIATE  = 1 << 3,
+	MG_ACTION_ASK_CONFIRM_UPDATE      = 1 << 4,
+	MG_ACTION_ASK_CONFIRM_DELETE      = 1 << 5,
+	MG_ACTION_ASK_CONFIRM_INSERT      = 1 << 6,
 
 	/* Error reporting */
-	MG_ACTION_REPORT_ERROR       = 1 << 6
+	MG_ACTION_REPORT_ERROR            = 1 << 7
+};
+
+/* enum for the different possible actions */
+enum _MgAction
+{
+	/* actions in MgWorkWidget widgets */
+	MG_ACTION_NEW_DATA,
+	MG_ACTION_WRITE_MODIFIED_DATA,
+	MG_ACTION_DELETE_SELECTED_DATA,
+	MG_ACTION_UNDELETE_SELECTED_DATA,
+	MG_ACTION_RESET_DATA,
+	MG_ACTION_MOVE_FIRST_RECORD,
+	MG_ACTION_MOVE_PREV_RECORD,
+	MG_ACTION_MOVE_NEXT_RECORD,
+	MG_ACTION_MOVE_LAST_RECORD,
+	MG_ACTION_MOVE_FIRST_CHUNCK,
+        MG_ACTION_MOVE_PREV_CHUNCK,
+        MG_ACTION_MOVE_NEXT_CHUNCK,
+        MG_ACTION_MOVE_LAST_CHUNCK
 };
 
 #endif
