@@ -62,6 +62,8 @@ static void        mg_join_replace_refs        (MgReferer *iface, GHashTable *re
 /* When the Query or any of the refering MgTarget is nullified */
 static void        nullified_object_cb (GObject *obj, MgJoin *join);
 static void        target_ref_lost_cb (MgRefBase *ref, MgJoin *join);
+static void        target_removed_cb (MgQuery *query, MgTarget *target, MgJoin *join);
+
 
 #ifdef debug
 static void        mg_join_dump (MgJoin *join, guint offset);
@@ -260,6 +262,9 @@ mg_join_new_with_targets (MgQuery *query, MgTarget *target_1, MgTarget *target_2
 	g_signal_connect (G_OBJECT (query), "nullified",
 			  G_CALLBACK (nullified_object_cb), mg_join);
 
+	g_signal_connect (G_OBJECT (query), "target_removed",
+			  G_CALLBACK (target_removed_cb), mg_join);
+
 	/* if the references to the any target is lost then we want to nullify the join */
 	g_signal_connect (G_OBJECT (mg_join->priv->target1), "ref_lost",
 			  G_CALLBACK (target_ref_lost_cb), mg_join);
@@ -323,6 +328,9 @@ mg_join_new_with_xml_ids (MgQuery *query, const gchar *target_1_xml_id, const gc
 	g_signal_connect (G_OBJECT (query), "nullified",
 			  G_CALLBACK (nullified_object_cb), mg_join);
 
+	g_signal_connect (G_OBJECT (query), "target_removed",
+			  G_CALLBACK (target_removed_cb), mg_join);
+
 	/* if the references to the any target is lost then we want to nullify the join */
 	g_signal_connect (G_OBJECT (mg_join->priv->target1), "ref_lost",
 			  G_CALLBACK (target_ref_lost_cb), mg_join);
@@ -361,6 +369,9 @@ mg_join_new_copy (MgJoin *orig, GHashTable *replacements)
 	g_signal_connect (G_OBJECT (orig->priv->query), "nullified",
 			  G_CALLBACK (nullified_object_cb), mg_join);
 
+	g_signal_connect (G_OBJECT (orig->priv->query), "target_removed",
+			  G_CALLBACK (target_removed_cb), mg_join);
+
 	mg_join->priv->target1 = MG_REF_BASE (mg_ref_base_new_copy (orig->priv->target1));
 	mg_join->priv->target2 = MG_REF_BASE (mg_ref_base_new_copy (orig->priv->target2));
 	mg_join->priv->join_type = orig->priv->join_type;
@@ -395,6 +406,25 @@ target_ref_lost_cb (MgRefBase *ref, MgJoin *join)
 	mg_base_nullify (MG_BASE (join));
 }
 
+static void
+target_removed_cb (MgQuery *query, MgTarget *target, MgJoin *join)
+{
+	MgBase *t;
+	gboolean to_nullify = FALSE;
+
+	t = mg_ref_base_get_ref_object (join->priv->target1);
+	if (t == (MgBase *) target)
+		to_nullify = TRUE;
+	if (! to_nullify) {
+		t = mg_ref_base_get_ref_object (join->priv->target2);
+		if (t == (MgBase *) target)
+			to_nullify = TRUE;
+	}
+
+	if (to_nullify)
+		mg_base_nullify (MG_BASE (join));
+}
+
 static void
 nullified_cond_cb (MgCondition *cond, MgJoin *join) 
 {
@@ -421,6 +451,8 @@ mg_join_dispose (GObject *object)
 		if (mg_join->priv->query) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (mg_join->priv->query),
 							      G_CALLBACK (nullified_object_cb), mg_join);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (mg_join->priv->query),
+							      G_CALLBACK (target_removed_cb), mg_join);
 			mg_join->priv->query = NULL;
 		}
 		if (mg_join->priv->target1) {
@@ -639,7 +671,7 @@ mg_join_swap_targets (MgJoin *join)
  * query fields which are either of type MgQfField and reference one of the two
  * targets which @join uses, or any other query field type.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 mg_join_set_condition (MgJoin *join, MgCondition *cond)
