@@ -57,8 +57,6 @@ static gboolean    mg_qf_all_load_from_xml (MgXmlStorage *iface, xmlNodePtr node
 static void              mg_qf_all_field_init      (MgFieldIface *iface);
 static MgEntity         *mg_qf_all_get_entity      (MgField *iface);
 static MgServerDataType *mg_qf_all_get_data_type   (MgField *iface);
-static const gchar      *mg_qf_all_get_name        (MgField *iface);
-static const gchar      *mg_qf_all_get_description (MgField *iface);
 
 /* Renderer interface */
 static void            mg_qf_all_renderer_init      (MgRendererIface *iface);
@@ -189,8 +187,6 @@ mg_qf_all_field_init (MgFieldIface *iface)
 {
 	iface->get_entity = mg_qf_all_get_entity;
 	iface->get_data_type = mg_qf_all_get_data_type;
-	iface->get_name = mg_qf_all_get_name;
-	iface->get_description = mg_qf_all_get_description;
 }
 
 static void
@@ -553,23 +549,6 @@ mg_qf_all_get_data_type (MgField *iface)
 	return NULL;
 }
 
-static const gchar *
-mg_qf_all_get_name (MgField *iface)
-{
-	g_return_val_if_fail (iface && IS_MG_QF_ALL (iface), NULL);
-
-	return mg_base_get_name (MG_BASE (iface));
-}
-
-static const gchar *
-mg_qf_all_get_description (MgField *iface)
-{
-	g_return_val_if_fail (iface && IS_MG_QF_ALL (iface), NULL);
-	
-	return mg_base_get_description (MG_BASE (iface));	
-}
-
-
 /* 
  * MgXmlStorage interface implementation
  */
