@@ -1,6 +1,6 @@
 /* mg-ref-base.c
  *
- * Copyright (C) 2003 Vivien Malerba
+ * Copyright (C) 2003 - 2004 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,12 +27,14 @@
 #include "mg-db-field.h"
 #include "mg-query.h"
 #include "mg-target.h"
+#include "mg-condition.h"
 #include "mg-entity.h"
 #include "mg-field.h"
 #include "mg-qfield.h"
 #include "mg-server.h"
 #include "mg-server-function.h"
 #include "mg-server-aggregate.h"
+#include "mg-custom-layout.h"
 
 #include "mg-qf-all.h"
 #include "mg-qf-field.h"
@@ -371,7 +373,8 @@ handled_object_type (GType type)
 	    (type == MG_FIELD_TYPE) ||
 	    (type == MG_SERVER_FUNCTION_TYPE) ||
 	    (type == MG_SERVER_AGGREGATE_TYPE) ||
-	    (type == MG_QFIELD_TYPE))
+	    (type == MG_QFIELD_TYPE) ||
+	    (type == MG_CUSTOM_LAYOUT_TYPE))
 		retval = type;
 
 	/* type conversion */
@@ -387,7 +390,7 @@ handled_object_type (GType type)
 /**
  * mg_ref_base_set_ref_name
  * @ref: a #MgRefBase object
- * @ref_type: the requested referenced object's data type
+ * @ref_type: the requested referenced object's data type, or 0 if not specified and @type = REFERENCE_BY_XML_ID
  * @type: how to interpret the @name argument
  * @name: the name of the requested object
  *
@@ -401,8 +404,43 @@ mg_ref_base_set_ref_name (MgRefBase *ref, GType ref_type, MgRefBaseType type, co
 {
 	g_return_if_fail (ref && IS_MG_REF_BASE (ref));
 	g_return_if_fail (ref->priv);
+	g_return_if_fail (name && *name);
 
 	/* make sure we know how to retreive the requested object */
+	if (!ref_type && (type == REFERENCE_BY_XML_ID) && (strlen (name) > 2)) {
+		gchar *str = g_strdup (name), *ptr, *tok;
+		gboolean has_sep = FALSE; /* TRUE if there is a ":" in the string */
+
+		ptr = strtok_r (str, ":", &tok);
+		ptr = strtok_r (NULL, ":", &tok);
+		if (!ptr)
+			ptr = str;
+		else
+			has_sep = TRUE;
+		if ((strlen (ptr) > 2) || has_sep) {
+			if ((*ptr == 'T') && (*(ptr+1) == 'V'))
+				ref_type = MG_DB_TABLE_TYPE;
+			if ((*ptr == 'F') && (*(ptr+1) == 'I'))
+				ref_type = MG_DB_FIELD_TYPE;
+			if ((*ptr == 'D') && (*(ptr+1) == 'T'))
+				ref_type = MG_SERVER_DATA_TYPE_TYPE;
+			if ((*ptr == 'P') && (*(ptr+1) == 'R'))
+				ref_type = MG_SERVER_FUNCTION_TYPE;
+			if ((*ptr == 'A') && (*(ptr+1) == 'G'))
+				ref_type = MG_SERVER_AGGREGATE_TYPE;
+			if ((*ptr == 'Q') && (*(ptr+1) == 'U'))
+				ref_type = MG_QUERY_TYPE;
+			if ((*ptr == 'Q') && (*(ptr+1) == 'F'))
+				ref_type = MG_QFIELD_TYPE;
+			if (has_sep && (*ptr == 'T'))
+				ref_type = MG_TARGET_TYPE;
+			if (has_sep && (*ptr == 'C'))
+				ref_type = MG_CONDITION_TYPE;
+			if ((*ptr == 'C') && (*(ptr+1) == 'L'))
+				ref_type = MG_CUSTOM_LAYOUT_TYPE;
+		}
+		g_free (str);
+	}
 	ref_type = handled_object_type (ref_type);
 	g_return_if_fail (ref_type);
 
@@ -530,7 +568,7 @@ mg_ref_base_replace_ref_object (MgRefBase *ref, GHashTable *replacements)
  * mg_ref_base_get_ref_name
  * @ref: a #MgRefBase object
  * @ref_type: where to store the requested referenced object's data type, or NULL
- * @type: where to store how to interpret the @name argument, or NULL
+ * @type: where to store how to interpret the returned name, or NULL
  *
  * Get the caracteristics of the requested object
  *
@@ -777,6 +815,18 @@ mg_ref_base_activate (MgRefBase *ref)
 			obj = MG_BASE (agg);
 	}
 
+	/* Custom Layout */
+	if (!done && (ref->priv->requested_type == MG_CUSTOM_LAYOUT_TYPE)) {
+		MgCustomLayout *layout = NULL;
+
+		done = TRUE;
+		if (ref->priv->ref_type == REFERENCE_BY_XML_ID)
+			layout = mg_conf_get_layout_by_xml_id (mg_base_get_conf (MG_BASE (ref)), ref->priv->name);
+		else
+			TO_IMPLEMENT; /* not really needed, anyway */
+		if (layout)
+			obj = MG_BASE (layout);
+	}
 
 	/* Object treatment */
 	if (obj) {
@@ -857,23 +907,25 @@ mg_ref_base_is_active (MgRefBase *ref)
 static void
 mg_ref_base_dump (MgRefBase *ref, guint offset)
 {
+	gchar *str;
+	gint i;
+
 	g_return_if_fail (ref && IS_MG_REF_BASE (ref));
 	g_return_if_fail (ref->priv);
 
-	if (ref->priv->ref_object) 
-		mg_base_dump (MG_BASE (ref->priv->ref_object), offset);
-	else {
-		gchar *str;
-		gint i;
+	/* string for the offset */
+	str = g_new0 (gchar, offset+1);
+	for (i=0; i<offset; i++)
+		str[i] = ' ';
+	str[offset] = 0;
 
-		/* string for the offset */
-		str = g_new0 (gchar, offset+1);
-		for (i=0; i<offset; i++)
-			str[i] = ' ';
-		str[offset] = 0;
-
-		g_print ("%s" D_COL_ERR "BaseRef to id '%s' not active\n" D_COL_NOR, str, ref->priv->name);
+	if (ref->priv->ref_object) {
+		g_print ("%s" D_COL_H1 "MgRefBase" D_COL_NOR " Active, points to id='%s': %p\n", str, 
+			 ref->priv->name, ref->priv->ref_object);
+		/*mg_base_dump (MG_BASE (ref->priv->ref_object), offset);*/
 	}
+	else 
+		g_print ("%s" D_COL_ERR "BaseRef to id '%s' not active\n" D_COL_NOR, str, ref->priv->name);
 }
 #endif
 
