@@ -704,6 +704,8 @@ mg_target_load_from_xml (MgXmlStorage *iface, xmlNodePtr node, GError **error)
 				target_ref = MG_QUERY_TYPE;
 			mg_ref_base_set_ref_name (target->priv->entity_ref, target_ref, 
 						  REFERENCE_BY_XML_ID, str);
+			mg_base_set_name (MG_BASE (target), 
+					  mg_base_get_name (mg_ref_base_get_ref_object (target->priv->entity_ref)));
 			name = TRUE;
 		}
 		g_free (str);
