@@ -1,6 +1,6 @@
 /* mg-work-core.c
  *
- * Copyright (C) 2002 Vivien Malerba
+ * Copyright (C) 2002 - 2004 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,21 +47,23 @@ static void mg_work_core_dispose (GObject *object);
 static void mg_work_core_initialize (MgWorkCore *core);
 
 struct _MgWorkCorePriv {
-	/* copy of query_select to chich various fields have been added */
-        MgQuery           *query_select_improved;
+	/* copy of query_select to which various fields have been added */
+        MgQuery           *query_select_hybrid;
 
 	/* objects replacements corresponding to queries being copied */
         GHashTable        *replacements_query_select; /* from the 'query' arg of _new() to 'query_select' */
-        GHashTable        *replacements_query_select_improved; /* from the 'query_select' to 'query_select_improved' */
+        GHashTable        *replacements_query_select_hybrid; /* from the 'query_select' to 'query_select_hybrid' */
 
-	/* list of MgQfValue fields added to 'query_select_improved' to express a condition in the modif queries */
+	GHashTable        *target_to_ref_pk_field; /* */
+
+	/* list of MgQfValue fields added to 'query_select_hybrid' to express a condition in the modif queries */
 	GSList            *for_cond_value_qfields;
 
-	/* list of ('query_select_improved') fields which have been added when using FK constraints */
+	/* list of ('query_select_hybrid') fields which have been added when using FK constraints */
 	GSList            *ref_integ_fields;
 
 	/* performances improvement */
-	GHashTable        *nodes_hash; /* key=node->param, value=node */
+	GHashTable        *nodes_hash; /* key=node->param, value= MgWorkCoreNode node */
 };
 
 /* 
@@ -154,9 +156,10 @@ mg_work_core_init (MgWorkCore * core)
 	core->no_show_params = NULL;
 
 	core->priv = g_new0 (MgWorkCorePriv, 1);
-	core->priv->query_select_improved = NULL;
+	core->priv->query_select_hybrid = NULL;
 	core->priv->replacements_query_select = NULL;
-	core->priv->replacements_query_select_improved = NULL;
+	core->priv->replacements_query_select_hybrid = NULL;
+	core->priv->target_to_ref_pk_field = g_hash_table_new (NULL, NULL);
 	core->priv->for_cond_value_qfields = NULL;
 	core->priv->ref_integ_fields = NULL;
 	core->priv->nodes_hash = NULL;
@@ -198,6 +201,7 @@ mg_work_core_new (MgQuery *query, MgTarget *modified)
 
 	replacements = g_hash_table_new (NULL, NULL);
 	core->query_select = MG_QUERY (mg_query_new_copy (query, replacements));
+	mg_query_order_fields_using_join_conds (core->query_select);
 	g_signal_connect (G_OBJECT (core->query_select), "nullified",
 			  G_CALLBACK (nullified_query_cb), core);
 
@@ -223,8 +227,8 @@ nullified_query_cb (MgQuery *query, MgWorkCore *core)
 	if (query == core->query_select) 
 		core->query_select = NULL;
 
-	if (core->priv && (query == core->priv->query_select_improved)) 
-		core->priv->query_select_improved = NULL;
+	if (core->priv && (query == core->priv->query_select_hybrid)) 
+		core->priv->query_select_hybrid = NULL;
 
 	if (query == core->query_update) 
 		core->query_update = NULL;
@@ -279,17 +283,22 @@ mg_work_core_dispose (GObject *object)
 
 	/* hash tables */
 	if (core->priv) {
-		if (core->priv->query_select_improved)
-			nullified_query_cb (core->priv->query_select_improved, core);
+		if (core->priv->query_select_hybrid)
+			nullified_query_cb (core->priv->query_select_hybrid, core);
 
 		if (core->priv->replacements_query_select) {
 			g_hash_table_destroy (core->priv->replacements_query_select);
 			core->priv->replacements_query_select = NULL;
 		}
 		
-		if (core->priv->replacements_query_select_improved) {
-			g_hash_table_destroy (core->priv->replacements_query_select_improved);
-			core->priv->replacements_query_select_improved = NULL;
+		if (core->priv->replacements_query_select_hybrid) {
+			g_hash_table_destroy (core->priv->replacements_query_select_hybrid);
+			core->priv->replacements_query_select_hybrid = NULL;
+		}
+
+		if (core->priv->target_to_ref_pk_field) {
+			g_hash_table_destroy (core->priv->target_to_ref_pk_field);
+			core->priv->target_to_ref_pk_field = NULL;
 		}
 
 		if (core->priv->for_cond_value_qfields) {
@@ -385,37 +394,14 @@ mg_work_core_initialize (MgWorkCore *core)
 
 	/*
 	 * Making modifications to the SELECT query to:
-	 * -> expand any '*' into all the corresponding fields 
+	 * -> expand any '*' into all the corresponding fields and order by those fields
 	 * -> add PK fields for the entity which will be modified
 	 */
 	/* expand any '*' into all the corresponding fields */
-	fields = mg_entity_get_visible_fields (MG_ENTITY (core->query_select));
+	fields = mg_query_expand_all_field (core->query_select, NULL);
 	list = fields;
 	while (list) {
-		if (IS_MG_QF_ALL (list->data)) {
-			MgTarget *target;
-			GSList *entfields, *list2;
-			MgField *newfield;
-			
-			target = mg_qf_all_get_target (MG_QF_ALL (list->data));
-			entfields = mg_entity_get_visible_fields (mg_target_get_represented_entity (target));
-			list2 = entfields;
-			while (list2) {
-				newfield = MG_FIELD (mg_qf_field_new_with_objects (core->query_select,
-										   target, MG_FIELD (list2->data)));
-				mg_entity_add_field_before (MG_ENTITY (core->query_select), newfield,
-							    MG_FIELD (list->data));
-				mg_query_set_order_by_field (core->query_select, MG_QFIELD (newfield),
-							     G_MAXINT, TRUE);
-				mg_base_set_name (MG_BASE (newfield), mg_base_get_name (MG_BASE (list2->data)));
-				mg_base_set_description (MG_BASE (newfield), 
-							 mg_base_get_description (MG_BASE (list2->data)));
-				g_object_unref (G_OBJECT (newfield));
-				list2 = g_slist_next (list2);
-			}
-			g_slist_free (entfields);
-			mg_qfield_set_visible (MG_QFIELD (list->data), FALSE);
-		}
+		mg_query_set_order_by_field (core->query_select, MG_QFIELD (list->data), G_MAXINT, TRUE);
 		list = g_slist_next (list);
 	}
 	g_slist_free (fields);
@@ -437,7 +423,7 @@ mg_work_core_initialize (MgWorkCore *core)
 				list = fields;
 				while (list) {
 					if (! (pkf =  mg_query_get_any_field_by_ref_field (core->query_select, 
-											   MG_FIELD (list->data)))) {
+											   NULL, MG_FIELD (list->data)))) {
 						pkf = MG_QFIELD (mg_qf_field_new_with_objects (core->query_select, 
 											       core->modif_target,
 											       MG_FIELD (list->data)));
@@ -490,7 +476,7 @@ mg_work_core_initialize (MgWorkCore *core)
 	 */
 	if (core->modif_target) {
 		core->work_context = make_work_context_target (core);
-		top_query = core->priv->query_select_improved;
+		top_query = core->priv->query_select_hybrid;
 	}
 	else 
 		core->work_context = make_work_context_no_target (core);
@@ -535,9 +521,9 @@ mg_work_core_initialize (MgWorkCore *core)
 	g_print ("---------------------------------- >> QUERY SELECT ----------------------------------\n");
 	mg_base_dump (MG_BASE (core->query_select), 0);
 	g_print ("---------------------------------- << QUERY SELECT ----------------------------------\n");
-	g_print ("---------------------------------- >> QUERY SELECT IMPROVED ----------------------------------\n");
-	mg_base_dump (MG_BASE (core->priv->query_select_improved), 0);
-	g_print ("---------------------------------- << QUERY SELECT IMPROVED ----------------------------------\n");
+	g_print ("---------------------------------- >> QUERY SELECT HYBRID ----------------------------------\n");
+	mg_base_dump (MG_BASE (core->priv->query_select_hybrid), 0);
+	g_print ("---------------------------------- << QUERY SELECT HYBRID ----------------------------------\n");
 #endif
 	
 	/* 
@@ -545,7 +531,7 @@ mg_work_core_initialize (MgWorkCore *core)
 	 * widgets using 'core'
 	 * computing the core->no_show_params list 
 	 */
-	if (core->priv->query_select_improved) {
+	if (core->priv->query_select_hybrid) {
 		GSList *tmplist;
 
 		/* alias parameters */
@@ -564,7 +550,7 @@ mg_work_core_initialize (MgWorkCore *core)
 		}
 
 		/* condition parameters */
-		g_object_get (G_OBJECT (core->priv->query_select_improved), 
+		g_object_get (G_OBJECT (core->priv->query_select_hybrid), 
 			      "really_all_fields", &list, NULL);
 		while (list) {
 			if (g_slist_find (core->priv->for_cond_value_qfields, list->data)) {
@@ -583,7 +569,7 @@ mg_work_core_initialize (MgWorkCore *core)
 		}
 
 		/* params for internal fields */
-		tmplist = mg_entity_get_visible_fields (MG_ENTITY (core->priv->query_select_improved));
+		tmplist = mg_entity_get_visible_fields (MG_ENTITY (core->priv->query_select_hybrid));
 		list = tmplist;
 		while (list) {
 			if (mg_qfield_is_internal (MG_QFIELD (list->data)) &&
@@ -628,8 +614,9 @@ mg_work_core_initialize (MgWorkCore *core)
 					MgQfield *query_select_field;
 					
 					field = mg_qf_field_get_ref_field (MG_QF_FIELD (list2->data));
-					query_select_field = mg_query_get_visible_field_by_ref_field (core->query_select,
-												      field);
+					query_select_field = 
+						mg_query_get_visible_field_by_ref_field (core->query_select,
+											 NULL, field);
 					if (query_select_field) {
 						pos = mg_entity_get_field_index (MG_ENTITY (core->query_select), 
 										 MG_FIELD (query_select_field));
@@ -681,8 +668,8 @@ mg_work_core_initialize (MgWorkCore *core)
 				if (IS_MG_QF_FIELD (field1) && IS_MG_QF_VALUE (field2)) {
 					MgQfield *rfield1, *rfield2, *value_prov;	
 
-					rfield1 = g_hash_table_lookup (core->priv->replacements_query_select_improved, field1);
-					rfield2 = g_hash_table_lookup (core->priv->replacements_query_select_improved, field2);
+					rfield1 = g_hash_table_lookup (core->priv->replacements_query_select_hybrid, field1);
+					rfield2 = g_hash_table_lookup (core->priv->replacements_query_select_hybrid, field2);
 					g_object_get (G_OBJECT (rfield1), "value_provider", &value_prov, NULL);
 
 					if (value_prov && rfield2) {
@@ -716,7 +703,7 @@ make_query_update (MgWorkCore *core)
 	MgConf *conf = mg_base_get_conf (MG_BASE (core->query_select));
 	MgCondition *cond = NULL;
 	GHashTable *replacements;
-	MgQuery *top_query = core->priv->query_select_improved;
+	MgQuery *top_query = core->priv->query_select_hybrid;
 
 	/* Query object */
 	query = MG_QUERY (mg_query_new (conf));
@@ -814,7 +801,7 @@ make_query_update (MgWorkCore *core)
 	g_signal_connect (G_OBJECT (core->query_update), "nullified",
 			  G_CALLBACK (nullified_query_cb), core);
 
-#ifdef debug
+#ifdef debug_NO
 	{
 		gchar *sql;
 		sql = mg_renderer_render_as_sql (MG_RENDERER (query), NULL, 0, NULL);
@@ -835,7 +822,7 @@ make_query_delete (MgWorkCore *core)
 	MgQuery *query;
 	MgConf *conf = mg_base_get_conf (MG_BASE (core->query_select));
 	MgCondition *cond = NULL;
-	MgQuery *top_query = core->priv->query_select_improved;
+	MgQuery *top_query = core->priv->query_select_hybrid;
 
 	/* Query object */
 	query = MG_QUERY (mg_query_new (conf));
@@ -885,7 +872,7 @@ make_query_delete (MgWorkCore *core)
 		list = g_slist_next (list);
 	}
 
-#ifdef debug
+#ifdef debug_NO
 	{
 		gchar *sql;
 		sql = mg_renderer_render_as_sql (MG_RENDERER (query), NULL, 0, NULL);
@@ -906,7 +893,7 @@ make_query_insert (MgWorkCore *core)
 	MgTarget *target;
 	MgQuery *query;
 	MgConf *conf = mg_base_get_conf (MG_BASE (core->query_select));
-	MgQuery *top_query = core->priv->query_select_improved;
+	MgQuery *top_query = core->priv->query_select_hybrid;
 
 	/* Query object */
 	query = MG_QUERY (mg_query_new (conf));
@@ -953,7 +940,7 @@ make_query_insert (MgWorkCore *core)
 		list = g_slist_next (list);
 	}
 
-#ifdef debug
+#ifdef debug_NO
 	{
 		gchar *sql;
 		sql = mg_renderer_render_as_sql (MG_RENDERER (query), NULL, 0, NULL);
@@ -1051,7 +1038,7 @@ make_work_context_target (MgWorkCore *core)
 	targets_h = g_hash_table_new (NULL, NULL);
 
 	/*
-	 * make a copy of 'core->query_select'
+	 * make a copy of 'core->query_select' into 'top_query'
 	 */
 	replacements = g_hash_table_new (NULL, NULL);
 	top_query = MG_QUERY (mg_query_new_copy (core->query_select, replacements));
@@ -1068,6 +1055,16 @@ make_work_context_target (MgWorkCore *core)
 	}
 	core->priv->ref_integ_fields = g_slist_concat (core->priv->ref_integ_fields, fields);
 
+	/*
+	 * POSSIBLE IMPROVEMENT:
+	 * Parse top_query's joins which have one target corresponding to the modif target,
+	 * and if one of the fields used by the join condition is not visible, then move it
+	 * close to the one which is visible.
+	 *
+	 * The problem is that the parameter for "fk_field" of such a join is set close to the "fk_field" and usually
+	 * we want it at the same place as what gets displayed in the SELECT (which is the "ref_pk_field").
+	 */
+
 	/*
 	 * For each MgQfValue in 'top_query', we need to remove the "value_provider" property if it exists because
 	 * these parameters are only aliases for parameters we have in the 'core->args_context' and we don't 
@@ -1084,7 +1081,7 @@ make_work_context_target (MgWorkCore *core)
 	 * make a list ('sel_queries') of SELECT queries: one for each target in the 'core->query_select'
 	 * query. Only the MgQfield of type MG_QF_FIELD are inserted into the new queries.
 	 *
-	 * For the 'core->modif_target', we don't start from scratch, but from the query created just above (= 'top_query')
+	 * For the 'core->modif_target', we don't start from scratch, but from the 'top_query' query created just above
 	 */
 	fields = mg_entity_get_visible_fields (MG_ENTITY (core->query_select));
 	list = fields;
@@ -1146,7 +1143,7 @@ make_work_context_target (MgWorkCore *core)
 				realfield = g_hash_table_lookup (replacements, list->data);
 
 				vfield = MG_QF_VALUE (mg_qf_value_new (query, mg_field_get_data_type (realfield)));
-				mg_entity_add_field (MG_ENTITY (query), MG_FIELD (vfield));
+				mg_entity_add_field_before (MG_ENTITY (query), MG_FIELD (vfield), MG_FIELD (realfield));
 				mg_base_set_name (MG_BASE (vfield), mg_base_get_name (MG_BASE (realfield)));
 				mg_base_set_description (MG_BASE (vfield), mg_base_get_description (MG_BASE (realfield)));
 				g_object_unref (G_OBJECT (vfield));
@@ -1165,7 +1162,7 @@ make_work_context_target (MgWorkCore *core)
 					if (mg_db_field_get_default_value (dbfield))
 						mg_qf_value_set_default_value (vfield, 
 									       mg_db_field_get_default_value (dbfield));
-					
+
 					srv = mg_conf_get_server (mg_base_get_conf (MG_BASE (core->query_select)));
 					default_dh = mg_server_get_object_handler (srv, G_OBJECT (dbfield));
 				}
@@ -1183,12 +1180,7 @@ make_work_context_target (MgWorkCore *core)
 						g_object_set (G_OBJECT (vfield), "handler_plugin", plugin, NULL);
 					}
 				}
-				
-				reffield = mg_qf_field_get_ref_field (MG_QF_FIELD (realfield));
-				if (IS_MG_DB_FIELD (reffield)) 
-					mg_qf_value_set_not_null (vfield,
-								  !mg_db_field_is_null_allowed (MG_DB_FIELD (reffield)));
-				
+								
 				/* keep a reference to the field in 'core->query_select' which we are now treating */
 				g_object_set_data (G_OBJECT (vfield), "position", list->data);
 			}
@@ -1268,8 +1260,9 @@ make_work_context_target (MgWorkCore *core)
 
 				if (IS_MG_QF_FIELD (fields->data)) {
 					MgField *ref_field = mg_qf_field_get_ref_field (MG_QF_FIELD (fields->data));
-					MgField *realfield = (MgField*) mg_query_get_any_field_by_ref_field (sel_query, 
-													     ref_field);
+					MgField *realfield = 
+						(MgField*) mg_query_get_any_field_by_ref_field (sel_query, 
+												NULL, ref_field);
 
 					if (!realfield) {
 						gchar *str;
@@ -1423,7 +1416,7 @@ make_work_context_target (MgWorkCore *core)
 			
 		list = g_slist_next (list);
 	}
-	core->priv->replacements_query_select_improved = replacements;
+	core->priv->replacements_query_select_hybrid = replacements;
 
 	/*
 	 * create MgWorkCoreNode nodes 
@@ -1489,8 +1482,8 @@ make_work_context_target (MgWorkCore *core)
 		if (list->data != top_query)
 			g_object_unref (G_OBJECT (list->data));
 		else {
-			core->priv->query_select_improved = top_query;
-			g_signal_connect (G_OBJECT (core->priv->query_select_improved), "nullified",
+			core->priv->query_select_hybrid = top_query;
+			g_signal_connect (G_OBJECT (core->priv->query_select_hybrid), "nullified",
 					  G_CALLBACK (nullified_query_cb), core);
 		}
 		list = g_slist_next (list);
@@ -1503,8 +1496,8 @@ make_work_context_target (MgWorkCore *core)
 	/*
 	 * computes the core->params_modif_queries_value_providers list
 	 */
-	if (core->priv->query_select_improved) {
-		g_object_get (G_OBJECT (core->priv->query_select_improved), "really_all_fields", &list, NULL);
+	if (core->priv->query_select_hybrid) {
+		g_object_get (G_OBJECT (core->priv->query_select_hybrid), "really_all_fields", &list, NULL);
 		while (list) {
 			if (IS_MG_QF_FIELD (list->data)) {
 				if (mg_qf_field_get_target (MG_QF_FIELD (list->data)) == 
@@ -1631,7 +1624,7 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 	gchar *str;
 
 	/* adding PK fields if necessary to 'core->query_select'*/
-	tmp = mg_query_get_any_field_by_ref_field (core->query_select, ref_pk_field);
+	tmp = mg_query_get_any_field_by_ref_field (core->query_select, NULL, ref_pk_field);
 	if (!tmp) {
 		MgTarget *newtarget;
 		newtarget = dep->target;
@@ -1644,17 +1637,14 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 		mg_base_set_name (MG_BASE (query_sel_field), "_FKcons: PK field");
 		core->priv->ref_integ_fields = g_slist_prepend (core->priv->ref_integ_fields, query_sel_field);
 	}
-	else {
+	else 
 		query_sel_field = MG_QF_FIELD (tmp);
-		if (! mg_qfield_is_visible (MG_QFIELD (tmp))) {
-			mg_qfield_set_visible (MG_QFIELD (tmp), TRUE);
-			mg_qfield_set_internal (MG_QFIELD (tmp), TRUE);
-		}
-	}
+
 	mg_qfield_set_visible (MG_QFIELD (query_sel_field), TRUE);
+	mg_qfield_set_internal (MG_QFIELD (query_sel_field), TRUE);
 
 	/* adding PK fields if necessary to query_ref_pk */
-	tmp = mg_query_get_any_field_by_ref_field (query_ref_pk, ref_pk_field);
+	tmp = mg_query_get_any_field_by_ref_field (query_ref_pk, NULL, ref_pk_field);
 	if (!tmp) {
 		MgTarget *newtarget;
 		newtarget = g_object_get_data (G_OBJECT (query_ref_pk), "for_target");
@@ -1677,7 +1667,7 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 	qfield_pk = qfield;
 
 	/* adding FK fields if necessary to 'core->query_select'*/
-	tmp = mg_query_get_any_field_by_ref_field (core->query_select, fk_field);
+	tmp = mg_query_get_any_field_by_ref_field (core->query_select, NULL, fk_field);
 	if (!tmp) {
 		MgTarget *newtarget;
 		newtarget = dep->dependant->target;
@@ -1695,7 +1685,7 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 	mg_qfield_set_visible (MG_QFIELD (query_sel_field), TRUE);
 
 	/* adding FK fields if necessary to query_fk */
-	tmp = mg_query_get_any_field_by_ref_field (query_fk, fk_field);
+	tmp = mg_query_get_any_field_by_ref_field (query_fk, NULL, fk_field);
 	if (!tmp) {
 		MgTarget *newtarget;
 		newtarget = g_object_get_data (G_OBJECT (query_fk), "for_target");
@@ -1725,7 +1715,7 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 
 	/* add MgQfValue for this field */
 	vfield = MG_QF_VALUE (mg_qf_value_new (query_fk, mg_field_get_data_type (fk_field)));
-	mg_entity_add_field (MG_ENTITY (query_fk), MG_FIELD (vfield));
+	mg_entity_add_field_before (MG_ENTITY (query_fk), MG_FIELD (vfield), MG_FIELD (qfield));
 	g_object_unref (G_OBJECT (vfield));
 	mg_qf_value_set_is_parameter (vfield, TRUE);
 	mg_qfield_set_visible (MG_QFIELD (vfield), FALSE);
@@ -1775,6 +1765,10 @@ improve_queries_with_db_fields (MgWorkCore *core, TargetDep *dep,
 	/* keep a reference to the field in 'core->query_select' which we 
 	   are now treating */
 	g_object_set_data (G_OBJECT (vfield), "position", query_sel_field);
+
+
+	/* */
+	g_hash_table_insert (core->priv->target_to_ref_pk_field, dep->target, vfield);
 }
 
 static void
@@ -1820,7 +1814,6 @@ improve_queries_with_join_condition (MgWorkCore *core, TargetDep *dep,
 	list = mainconds;
 	while (list) {
 		MgCondition *cond = MG_CONDITION (list->data);
-		MgTarget *target;
 		MgQfield *field;
 		MgField *fk_field = NULL, *ref_pk_field = NULL;
 
@@ -1828,14 +1821,14 @@ improve_queries_with_join_condition (MgWorkCore *core, TargetDep *dep,
 		
 		field = mg_condition_leaf_get_operator (cond, MG_CONDITION_OP_LEFT);
 		g_return_if_fail (field && IS_MG_QF_FIELD (field));
-		if (mg_qf_field_get_target (field) == fk_target)
+		if (mg_qf_field_get_target (MG_QF_FIELD (field)) == fk_target)
 			fk_field = mg_qf_field_get_ref_field (MG_QF_FIELD (field));
 		else
 			ref_pk_field = mg_qf_field_get_ref_field (MG_QF_FIELD (field));
 		
 		field = mg_condition_leaf_get_operator (cond, MG_CONDITION_OP_RIGHT);
 		g_return_if_fail (field && IS_MG_QF_FIELD (field));
-		if (mg_qf_field_get_target (field) == fk_target)
+		if (mg_qf_field_get_target (MG_QF_FIELD (field)) == fk_target)
 			fk_field = mg_qf_field_get_ref_field (MG_QF_FIELD (field));
 		else
 			ref_pk_field = mg_qf_field_get_ref_field (MG_QF_FIELD (field));
@@ -2024,7 +2017,7 @@ make_target_deps_free (TargetDep *dep)
  *
  * Runs the @core->selec_query SELECT query and stores the resultset in @core->data_rs
  *
- * Returns: TRUE if not error occured
+ * Returns: TRUE if not error occurred
  */
 gboolean
 mg_work_core_run_select_query (MgWorkCore *core, GError **error)
@@ -2077,49 +2070,58 @@ mg_work_core_run_select_query (MgWorkCore *core, GError **error)
 /**
  * mg_work_core_find_param
  * @core: a #MgWorkCore object
- * @field: a #MgQfield which belongs to the SELECT query given as 'query' argument to mg_work_core_new()
+ * @field: a #MgQfield which belongs to the SELECT query given as 'query' argument to mg_work_core_new(), or
+ *         to @core->query_select (which is a copy of the query given as 'query' argument to mg_work_core_new())
+ * @in_exec_context: TRUE if to search in the exec context, or FALSE to search among the "data" context
  *
- * Finds the #MgParameter 'attached' to @field. First the parameters in 'args_context' are looked for, then 
- * the ones in 'work_context'.
+ * Finds the #MgParameter 'attached' to the values of @field. The @in_exec_context specifies if the 
+ * parameter is to be searched among the required parameters to execute the SELECT query 
+ * given as 'query' argument to mg_work_core_new(), or among the results of the execution of that query.
  *
  * Returns: the requested #MgParameter, or %NULL
  */
 MgParameter *
-mg_work_core_find_param (MgWorkCore *core, MgQfield *field)
+mg_work_core_find_param (MgWorkCore *core, MgQfield *field, gboolean in_exec_context)
 {
 	gpointer q_sel_field;
-	MgParameter *param;
+	MgParameter *param = NULL;
 
 	g_return_val_if_fail (core && IS_MG_WORK_CORE (core), NULL);
 	g_return_val_if_fail (field && IS_MG_QFIELD (field), NULL);
 
 	q_sel_field = g_hash_table_lookup (core->priv->replacements_query_select, field);
-	g_return_val_if_fail (q_sel_field, NULL);
+	if (!q_sel_field)
+		/* @field already belongs to @core->query_select */
+		q_sel_field = (gpointer) field;
 
-	/* first try in 'args_context' */
-	param = mg_context_find_parameter_for_field (core->args_context, MG_QFIELD (q_sel_field));
-	if (param) 
-		return param;
-	
-	/* if not found, then try in 'work_context' */
-	if (core->modif_target) {
-		gpointer q_sel_impr_field;
-		gpointer qf_value;
-
-		q_sel_impr_field = g_hash_table_lookup (core->priv->replacements_query_select_improved, q_sel_field);
-		g_return_val_if_fail (q_sel_impr_field, NULL);
-
-		param = mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (q_sel_field));
-		if (param)
-			return param;
-
-		g_object_get (G_OBJECT (q_sel_impr_field), "value_provider", &qf_value, NULL);
-		g_return_val_if_fail (qf_value, NULL);
-		
-		return mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (qf_value));
+	if (in_exec_context) {
+		/* try in 'args_context' */
+		param = mg_context_find_parameter_for_field (core->args_context, MG_QFIELD (q_sel_field));
+		if (!param && (q_sel_field != (gpointer) field))
+			param = mg_context_find_parameter_for_field (core->args_context, field);
 	}
-	else 
-		return mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (q_sel_field));
+	else {
+		/* try in 'work_context' */
+		if (core->modif_target) {
+			gpointer q_sel_impr_field;
+			gpointer qf_value;
+			
+			q_sel_impr_field = g_hash_table_lookup (core->priv->replacements_query_select_hybrid, 
+								q_sel_field);
+			g_return_val_if_fail (q_sel_impr_field, NULL);
+			
+			param = mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (q_sel_field));
+			if (!param) {
+				g_object_get (G_OBJECT (q_sel_impr_field), "value_provider", &qf_value, NULL);
+				if (qf_value)
+					param = mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (qf_value));
+			}
+		}
+		else 
+			param = mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (q_sel_field));
+	}
+	
+	return param;
 }
 
 
@@ -2130,7 +2132,7 @@ mg_work_core_find_param (MgWorkCore *core, MgQfield *field)
  *
  * Find the #MgWorkCoreNode node which handles @param.
  *
- * Returns:
+ * Returns: the corresponding #MgWorkCoreNode
  */
 MgWorkCoreNode *
 mg_work_core_find_core_node (MgWorkCore *core, MgParameter *param)
@@ -2140,3 +2142,55 @@ mg_work_core_find_core_node (MgWorkCore *core, MgParameter *param)
 	return g_hash_table_lookup (core->priv->nodes_hash, param);
 }
 
+
+/**
+ * mg_work_core_find_context_node
+ * @core: a #MgWorkCoreNode object
+ * @field:a #MgQfield which belongs to the SELECT query given as 'query' argument to mg_work_core_new(), or
+ *         to @core->query_select (which is a copy of the query given as 'query' argument to mg_work_core_new())
+ *
+ * Depending on if there is a target to be modified of not, the data for the @field field
+ * will be presented to the user as a simple entry area, or as a combo box. This function makes it
+ * possible to get the #MgContextNode where the data pointer by @field will be "used" (either through a 
+ * #MgParameter for simple entry area, or in a SELECT query for combo boxes).
+ *
+ * Returns: the #MgContextNode, or %NULL if not found.
+ */
+MgContextNode *
+mg_work_core_find_context_node (MgWorkCore *core, MgQfield *field)
+{
+	MgContextNode *cnode = NULL;
+	MgParameter *param;
+
+	g_return_val_if_fail (core && IS_MG_WORK_CORE (core), NULL);
+	g_return_val_if_fail (core->priv, NULL);
+
+	param = mg_work_core_find_param (core, field, FALSE);
+	if (param)
+		cnode = mg_context_find_node_for_param (core->work_context, param);
+
+	if (cnode)
+		return cnode;
+	
+	if (core->modif_target) {
+		gpointer q_sel_field;
+
+		q_sel_field = g_hash_table_lookup (core->priv->replacements_query_select, field);
+		if (!q_sel_field)
+			/* @field already belongs to @core->query_select */
+			q_sel_field = (gpointer) field;
+		
+		
+		if (IS_MG_QF_FIELD (q_sel_field)) {
+			MgTarget *target = mg_qf_field_get_target (MG_QF_FIELD (q_sel_field));
+			q_sel_field = g_hash_table_lookup (core->priv->target_to_ref_pk_field, target);
+			if (q_sel_field) {
+				param = mg_context_find_parameter_for_field (core->work_context, MG_QFIELD (q_sel_field));
+				if (param)
+					cnode = mg_context_find_node_for_param (core->work_context, param);
+			}
+		}
+	}
+
+	return cnode;
+}
