@@ -144,7 +144,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	
 	/* Green color */
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#00cd66", color);
+	gdk_color_parse (MG_COLOR_NORMAL_NULL, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -152,7 +152,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	colors[0] = color;
 	
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#00ef77", color);
+	gdk_color_parse (MG_COLOR_PRELIGHT_NULL, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -162,7 +162,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	
 	/* Blue color */
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#6495ed", color);
+	gdk_color_parse (MG_COLOR_NORMAL_DEFAULT, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -170,7 +170,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	colors[2] = color;
 	
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#75a6fe", color);
+	gdk_color_parse (MG_COLOR_PRELIGHT_DEFAULT, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -180,7 +180,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	
 	/* Red color */
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#ff6a6a", color);
+	gdk_color_parse (MG_COLOR_NORMAL_INVALID, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -188,7 +188,7 @@ GdkColor **utility_entry_build_info_colors_array ()
 	colors[4] = color;
 	
 	color = g_new0 (GdkColor, 1);
-	gdk_color_parse ("#ff7b7b", color);
+	gdk_color_parse (MG_COLOR_PRELIGHT_INVALID, color);
 	if (!gdk_colormap_alloc_color (gtk_widget_get_default_colormap (), color, FALSE, TRUE)) {
 		g_free (color);
 		color = NULL;
@@ -662,6 +662,7 @@ utility_combo_initialize_core (MgConf *conf, MgContext *context, MgContextNode *
 	/* connect the parameters dependencies */
 	list = ccore->nodes;
 	while (list) {
+		/* FIXME: make sure we don't connect several times to the changes of one single param */
 		GSList *depend = mg_parameter_get_dependencies (COMBO_NODE (list->data)->param);
 		while (depend) {
 			g_signal_connect (G_OBJECT (depend->data), "changed",
@@ -785,8 +786,10 @@ utility_combo_compute_model (ComboCore *core)
 		if (!sql) {
 			valid_data = FALSE;
 			data_model = make_single_line_model (_("No value available"));
-			if (error)
+			if (error) {
+				g_warning ("COMBO Model update SQL execution error: %s", error->message);
 				g_error_free (error);
+			}
 		}
 		else {
 			rs = mg_server_do_query (srv, sql, MG_SERVER_QUERY_SQL, &error);
