@@ -15,7 +15,7 @@ gint main (int argc, char **argv) {
 
 
 	if (!mg_conf_load_xml_file (conf, "DATA_GRAPH.xml", &error)) {
-		g_print ("Error occured:\n\t%s\n", error->message);
+		g_print ("Error occurred:\n\t%s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 		exit (1);
@@ -31,7 +31,7 @@ gint main (int argc, char **argv) {
 	g_slist_free (tmplist);
 	g_print ("# Saving DATA_GRAPH.dot                                            #\n");
 	if (!mg_graphviz_save_file (graph, "DATA_GRAPH.dot", &error)) {
-		g_print ("Error occured:\n\t%s\n", error->message);
+		g_print ("Error occurred:\n\t%s\n", error->message);
 		g_error_free (error);
 		error = NULL;
 		exit (1);
